SystemOrganization addCategory: #'SOM-Benchmarks'!
SystemOrganization addCategory: 'SOM-Benchmarks-DeltaBlue'!
SystemOrganization addCategory: 'SOM-Benchmarks-GraphSearch'!
SystemOrganization addCategory: 'SOM-Benchmarks-Json'!
SystemOrganization addCategory: 'SOM-Benchmarks-LanguageFeatures'!
SystemOrganization addCategory: 'SOM-Benchmarks-NBody'!
SystemOrganization addCategory: 'SOM-Benchmarks-Richards'!

OrderedCollection subclass: #DBPlan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBPlan commentStamp: '<historical>' prior: 0!
A Plan is an ordered list of constraints to be executed in sequence to resatisfy all currently satisfiable constraints in the face of one or more changing inputs.!

!DBPlan methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:49'!
execute
	"Execute my constraints in order."

	self do: [: c | c execute].! !

Object subclass: #DBAbstractConstraint
	instanceVariableNames: 'strength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBAbstractConstraint commentStamp: '<historical>' prior: 0!
I am an abstract class representing a system-maintainable relationship (or "constraint") between a set of variables. I supply a strength instance variable; concrete subclasses provide a means of storing the constrained variables and other information required to represent a constraint.

Instance variables:
	strength			the strength of this constraint <Strength>!

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
addConstraint
	"Activate this constraint and attempt to satisfy it."

	self addToGraph.
	DBPlanner current incrementalAdd: self.! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
addToGraph
	"Add myself to the constraint graph."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision. The output of
	 the choosen method must not have the given mark and must have a
	 walkabout strength less than that of this constraint."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 14:54'!
destroyConstraint
	"Deactivate this constraint, remove it from the constraint graph,
	 possibly causing other constraints to be satisfied, and destroy it."

	(self isSatisfied) ifTrue: [DBPlanner current incrementalRemove: self].
	self removeFromGraph.
	"self release."! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
inputsDo: aBlock
	"Assume that I am satisfied. Evaluate the given block on all my current
	 input variables."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 14:55'!
inputsKnown: mark
	"Assume that I am satisfied. Answer true if all my current inputs are
	 known. A variable is known if either a) it is 'stay' (i.e. it is a
	 constant at plan execution time), b) it has the given mark (indicating
	 that it has been computed by a constraint appearing earlier in the
	 plan), or c) it is not determined by any constraint."

	self inputsDo:
		[ :v |
		 (v mark = mark or: [(v stay) or: [v determinedBy == nil]]) ifFalse:
			[^false]].
	^true! !

!DBAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:53'!
isInput
	"Normal constraints are not input constraints. An input constraint is
	 one that depends on external state, such as the mouse, the keyboard,
	 a clock, or some arbitrary piece of imperative code."

	^false! !

!DBAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:54'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
longPrintOn: aStream

	aStream nextPut: $(.
	self shortPrintOn: aStream.
	aStream space; nextPutAll: self strength printString.
	(self isSatisfied)
		ifTrue:
			[aStream cr; space; space; space.
			 self inputsDo:
				[ :in | aStream nextPutAll: 'v', in printString, ' '].
			aStream nextPutAll: '-> '.
			aStream nextPutAll: 'v', self output printString]
		ifFalse:
			[aStream space; nextPutAll: 'UNSATISFIED'].
	aStream nextPut: $); cr.! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
output
	"Answer my current output variable. Raise an error if I am not
	 currently satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:57'!
printOn: aStream

	self shortPrintOn: aStream! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:56'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
removeFromGraph
	"Remove myself from the constraint graph."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 14:56'!
satisfy: mark
	"Attempt to find a way to enforce this (still unsatisfied) constraint.
	 If successful, record the solution, perhaps modifying the current
	 dataflow graph.  Answer the constraint that this constraint overrides,
	 if there is one, or nil, if there isn't."

	| overridden out |
	self chooseMethod: mark.
	(self isSatisfied)
		ifTrue:			"constraint can be satisfied"
			["mark inputs to allow cycle detection in addPropagate"
			 self inputsDo: [ :in | in mark: mark].
			 out := self output.
			 overridden := out determinedBy.
			 (overridden == nil) ifFalse: [overridden markUnsatisfied].
			 out determinedBy: self.
			 (DBPlanner current addPropagate: self mark: mark) ifFalse:
				[self notify:
					('Cycle encountered adding:\   ',
					 self printString,
					 '\Constraint removed.') withCRs.
				 ^nil].
			 out mark: mark]
		ifFalse:			"constraint cannot be satisfied"
			[overridden := nil.
			 (strength sameAs: (DBStrength required)) ifTrue:
				[self notify: 'Failed to satisfy a required constraint']].
	^ overridden! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
shortPrintOn: aStream

	aStream nextPutAll: self class name, '(', self printString, ')'.! !

!DBAbstractConstraint methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:53'!
strength
	^ strength! !

!DBAbstractConstraint methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 14:52'!
strength: strengthSymbol
	strength := DBStrength of: strengthSymbol! !

DBAbstractConstraint subclass: #DBBinaryConstraint
	instanceVariableNames: 'direction v1 v2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBBinaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having two possible output variables.

Instance variables:
	v1, v2		possible output variables <Variable>
	direction		one of:
					#forward (v2 is output)
					#backward (	v1 is output)
					nil (not satisfied)!

!DBBinaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:21'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
	v2 addConstraint: self.
	direction := nil.! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
chooseMethod: mark
	"Decide if I can be satisfied and which way I should flow based on the relative strength of the variables I relate, and record that decision."

	(v1 mark == mark) ifTrue:		"forward or nothing"
		[((v2 mark ~= mark) and: [strength stronger: v2 walkStrength]) 
			ifTrue: [ ^ direction := #forward ]
			ifFalse: [ ^ direction := nil ] ].

	(v2 mark == mark) ifTrue:		"backward or nothing"
		[((v1 mark ~= mark) and: [strength stronger: v1 walkStrength])
			ifTrue: [ ^ direction := #backward ]
			ifFalse: [ ^ direction := nil ] ].

	"if we get here, neither variable is marked, so we have choice"
	(v1 walkStrength weaker: v2 walkStrength)
		ifTrue:
			[(strength stronger: v1 walkStrength)
				ifTrue: [ ^ direction := #backward ]
				ifFalse: [ ^ direction := nil ]]
		ifFalse:
			[(strength stronger: v2 walkStrength)
				ifTrue: [ ^ direction := #forward ]
				ifFalse: [ ^ direction := nil ]]. ! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	direction == #forward
		ifTrue: [ aBlock value: v1 ]
		ifFalse: [ aBlock value: v2 ].! !

!DBBinaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:21'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^ direction notNil! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	direction := nil.! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
output
	"Answer my current output variable."

	direction == #forward
		ifTrue: [ ^ v2 ]
		ifFalse: [ ^ v1 ]! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:02'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
         the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [in := v2. out := v1].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: (in stay).
	(out stay) ifTrue: [ self execute ].		"stay optimization"! !

!DBBinaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 14:58'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	direction := nil.! !

!DBBinaryConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:21'!
var: variable1 var: variable2 strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := DBStrength of: strengthSymbol.
	v1 := variable1.
	v2 := variable2.
	direction := nil.
	self addConstraint.! !

DBBinaryConstraint subclass: #DBEqualityConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBEqualityConstraint commentStamp: '<historical>' prior: 0!
I constrain two variables to have the same value: "v1 = v2".!

!DBEqualityConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:55'!
var: variable1 var: variable2 strength: strengthSymbol
	"Install a constraint with the given strength equating the given
	 variables."

	^(self new) var: variable1 var: variable2 strength: strengthSymbol! !

!DBEqualityConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:18'!
execute
	"Enforce this constraint. Assume that it is satisfied."
 
	(direction == #forward)
		ifTrue: [v2 value: v1 value]
		ifFalse: [v1 value: v2 value].! !

DBBinaryConstraint subclass: #DBScaleConstraint
	instanceVariableNames: 'offset scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBScaleConstraint commentStamp: '<historical>' prior: 0!
I relate two variables by the linear scaling relationship:
"v2 = (v1 * scale) + offset". Either v1 or v2 may be changed to maintain this
relationship but the scale factor and offset are considered read-only.

Instance variables:
	scale		scale factor input variable <Variable>
	offset		offset input variable <Variable>!

!DBScaleConstraint class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:32'!
var: src var: scale var: offset var: dst strength: strengthSymbol
	"Install a scale constraint with the given strength on the given variables."

	^(self new) src: src scale: scale offset: offset dst: dst strength: strengthSymbol! !

!DBScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:23'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
   v2 addConstraint: self.
   scale addConstraint: self.
   offset addConstraint: self.
   direction := nil.
! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:30'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	(direction == #forward)
		ifTrue: [v2 value: (v1 value * scale value) + offset value]
		ifFalse: [v1 value: (v2 value - offset value) // scale value].! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	(direction == #forward)
		ifTrue: [ aBlock value: v1.
					 aBlock value: scale.
					 aBlock value: offset]
		ifFalse: [ aBlock value: v2.
					  aBlock 	value: scale. 
					  aBlock 	value: offset].! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay', the value for the current output of this constraint. Assume this constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [out := v1. in := v2].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: ((in stay) and: [(scale stay) and: [offset stay]]).
	out stay ifTrue: [self execute].		"stay optimization"! !

!DBScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:24'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	(scale == nil) ifFalse: [scale removeConstraint: self].
	(offset == nil) ifFalse: [offset removeConstraint: self].
	direction := nil.
! !

!DBScaleConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:27'!
src: srcVar scale: scaleVar offset: offsetVar dst: dstVar strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := DBStrength of: strengthSymbol.
	v1 := srcVar.
	v2 := dstVar.
	scale := scaleVar.
	offset := offsetVar.
	direction := nil.
	self addConstraint.! !

DBAbstractConstraint subclass: #DBUnaryConstraint
	instanceVariableNames: 'output satisfied'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBUnaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having a single possible output variable.

Instance variables:
	output		possible output variable <Variable>
	satisfied		true if I am currently satisfied <Boolean>!

DBUnaryConstraint subclass: #DBEditConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBEditConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same. I have one method with zero inputs and one output, which does nothing. Planners may exploit the fact that, if I am satisfied, my output will not change during plan execution. This is called "stay optimization."!

!DBEditConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:16'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!DBEditConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
execute
	"Edit constraints do nothing."! !

!DBEditConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:18'!
isInput
	"I indicate that a variable is to be changed by imperative code."

	^true! !

DBUnaryConstraint subclass: #DBStayConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBStayConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same.
I have one method with zero inputs and one output, which does nothing. Planners
may exploit the fact that, if I am satisfied, my output will not change during
plan execution. This is called "stay optimization."!

!DBStayConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:17'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!DBStayConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:20'!
execute
	"Stay constraints do nothing."! !

!DBUnaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:16'!
addToGraph
	"Add myself to the constraint graph."

	output addConstraint: self.
	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision."

	satisfied :=
		output mark ~= mark and:
		[strength stronger: output walkStrength].! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
inputsDo: aBlock
	"I have no input variables."! !

!DBUnaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:15'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^satisfied! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
output
	"Answer my current output variable."

	^ output! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	output walkStrength: strength.
	output stay: self isInput not.
	output stay ifTrue: [self execute].	"stay optimization"! !

!DBUnaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:41'!
removeFromGraph
	"Remove myself from the constraint graph."

	(output == nil) ifFalse: [output removeConstraint: self].
	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:15'!
var: aVariable strength: strengthSymbol
	"Initialize myself with the given variable and strength."

	strength := DBStrength of: strengthSymbol.
	output := aVariable.
	satisfied := false.
	self addConstraint.! !

Object subclass: #DBPlanner
	instanceVariableNames: 'currentMark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

DBPlanner class
	instanceVariableNames: 'currentPlanner'!

!DBPlanner commentStamp: '<historical>' prior: 0!
This benchmark is an implementation of the DeltaBlue Constraint Solver described in `The DeltaBlue Algorithm: An Incremental Constraint Hierarchy Solver'', by Bjorn N. Freeman-Benson and John Maloney, Communications of the ACM, January 1990 (also as University of Washington TR 89-08-06).
!

DBPlanner class
	instanceVariableNames: 'currentPlanner'!

!DBPlanner class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 21:11'!
chainTest: n
        "Do chain-of-equality-constraints performance tests."
        | vars editConstraint plan planner |
        planner := DBPlanner new.
        vars := Array new: n+1.
        1 to: n+1 do: [:i | vars at: i put: DBVariable new ].

        "thread a chain of equality constraints through the variables"
        1 to: n do:
            [ :i | | v1 v2 |
             v1 := vars at: i.
             v2 := vars at: i + 1.
             DBEqualityConstraint var: v1 var: v2 strength: #required].

        DBStayConstraint var: vars last strength: #strongDefault.
        editConstraint := DBEditConstraint var: (vars first) strength: #preferred.
        plan := planner extractPlanFromConstraints: (Array with: editConstraint).
        1 to: 100 do: [ :v | 
            vars first value: v.
            plan execute.
            vars last value ~= v ifTrue: [self error: 'Chain test failed!!!!']].
        editConstraint destroyConstraint
  ! !

!DBPlanner class methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 11:16'!
current
	^ currentPlanner! !

!DBPlanner class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:39'!
new
	^ currentPlanner := super new! !

!DBPlanner class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:22'!
projectionTest: n
        "This test constructs a two sets of variables related to each other by
         a simple linear transformation (scale and offset)."

        | scale offset src dst planner dests |
        planner := DBPlanner new.
        dests := OrderedCollection new.
        scale := DBVariable value: 10.
        offset := DBVariable value: 1000.
        1 to: n do:
            [ :i |
            src := DBVariable value: i.
            dst := DBVariable value: i.
            dests add: dst.
            DBStayConstraint var: src strength: #default.
            DBScaleConstraint var: src var: scale var: offset var: dst strength: #required].

        planner changeVar: src newValue: 17.
        dst value ~= 1170 ifTrue: [self error: 'Projection test 1 failed!!!!'].

        planner changeVar: dst newValue: 1050.
        src value ~= 5 ifTrue: [self error: 'Projection test 2 failed!!!!'].

        planner changeVar: scale newValue: 5.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 1000)
                ifTrue: [self error: 'Projection test 3 failed!!!!']].

        planner changeVar: offset newValue: 2000.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 2000)
                ifTrue: [self error: 'Projection test 4 failed!!!!']]! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addConstraintsConsuming: v to: aCollection

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 (c == determiningC or: [c isSatisfied not]) ifFalse:
			[aCollection add: c]].! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addPropagate: c mark: mark
	"Recompute the walkabout strengths and stay flags of all variables
	 downstream of the given constraint and recompute the actual values
	 of all variables whose stay flag is true. If a cycle is detected,
	 remove the given constraint and answer false. Otherwise, answer true.

	 Details: Cycles are detected when a marked variable is encountered
	 downstream of the given constraint. The sender is assumed to have
	 marked the inputs of the given constraint with the given mark. Thus,
	 encountering a marked node downstream of the output constraint means
	 that there is a path from the constraint's output to one of its
	 inputs."

	| todo d |
	todo := OrderedCollection with: c.
	[todo isEmpty] whileFalse:
		[d := todo removeFirst.
		 (d output mark = mark) ifTrue:
			[self incrementalRemove: c.
			 ^ false].
		 d recalculate.
		 self addConstraintsConsuming: d output to: todo].
	^ true! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:37'!
changeVar: aVariable newValue: newValue

	| editConstraint plan |
	editConstraint := DBEditConstraint var: aVariable strength: #preferred.
	plan := self extractPlanFromConstraints: (Array with: editConstraint).
	10 timesRepeat: [
		aVariable value: newValue.
		plan execute].
	editConstraint destroyConstraint.! !

!DBPlanner methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:03'!
constraintsConsuming: v do: aBlock

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 ((c == determiningC) or: [c isSatisfied not]) ifFalse:
			[aBlock value: c]].! !

!DBPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:59'!
extractPlanFromConstraints: constraints
	"Extract a plan for resatisfaction starting from the outputs of the
	 given constraints, usually a set of input constraints."

	| sources |
	sources := OrderedCollection new.
	constraints do:
		[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]].
	^self makePlan: sources! !

!DBPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:00'!
extractPlanFromVariables: variables
	"Extract a plan from the dataflow graph having the given variables. It
	 is assumed that the given set of variables is complete, or at least
	 that it contains all the input variables."

	| sources |
	sources := OrderedCollection new.
	variables do:
		[: v |
		 (v constraints) do:
			[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]]].
	^self makePlan: sources! !

!DBPlanner methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 15:58'!
incrementalAdd: c
	"Attempt to satisfy the given constraint and, if successful,
	 incrementally update the dataflow graph.

	 Details: If satifying the constraint is successful, it may override a
	 weaker constraint on its output. The algorithm attempts to resatisfy
	 that constraint using some other method. This process is repeated
	 until either a) it reaches a variable that was not previously
	 determined by any constraint or b) it reaches a constraint that
	 is too weak to be satisfied using any of its methods. The variables
	 of constraints that have been processed are marked with a unique mark
	 value so that we know where we've been. This allows the algorithm to
	 avoid getting into an infinite loop even if the constraint graph has
	 an inadvertent cycle."

	| mark overridden |
	mark := self newMark.
	overridden := c satisfy: mark.
	[overridden == nil] whileFalse:
		[overridden := overridden satisfy: mark].! !

!DBPlanner methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:34'!
incrementalRemove: c
	"Entry point for retracting a constraint. Remove the given constraint,
	 which should be satisfied, and incrementally update the dataflow
	 graph.

	 Details: Retracting the given constraint may allow some currently
	 unsatisfiable downstream constraint be satisfied. We thus collect a
	 list of unsatisfied downstream constraints and attempt to satisfy
	 each one in turn. This list is sorted by constraint strength,
	 strongest first, as a heuristic for avoiding unnecessarily adding
	 and then overriding weak constraints."

	| out unsatisfied |
	out := c output.
	c markUnsatisfied.
	c removeFromGraph.
	unsatisfied := self removePropagateFrom: out.
	unsatisfied do: [: u | self incrementalAdd: u].! !

!DBPlanner methodsFor: 'initialize' stamp: 'StefanMarr 4/2/2015 15:57'!
initialize

	"super initialize."

	currentMark := 1.! !

!DBPlanner methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:35'!
makePlan: sources
	"Extract a plan for resatisfaction starting from the given satisfied
	 source constraints, usually a set of input constraints. This method
	 assumes that stay optimization is desired; the plan will contain only
	 constraints whose output variables are not stay. Constraints that do
	 no computation, such as stay and edit constraints, are not included
	 in the plan.

	 Details: The outputs of a constraint are marked when it is added to
	 the plan under construction. A constraint may be appended to the plan
	 when all its input variables are known. A variable is known if either
	 a) the variable is marked (indicating that has been computed by a
	 constraint appearing earlier in the plan), b) the variable is 'stay'
	 (i.e. it is a constant at plan execution time), or c) the variable
	 is not determined by any constraint. The last provision is for past
	 states of history variables, which are not stay but which are also
	 not computed by any constraint."

	| mark plan todo c |
	mark := self newMark.
	plan := DBPlan new.
	todo := sources.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 ((c output mark ~= mark) and:		"not in plan already and..."
		  [c inputsKnown: mark]) ifTrue:	"eligible for inclusion"
			[plan addLast: c.
			 c output mark: mark.
			 self addConstraintsConsuming: c output to: todo]].
	^ plan! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:37'!
newMark
	"Select a previously unused mark value.

	 Details: We just keep incrementing. If necessary, the counter will
	 turn into a LargePositiveInteger. In that case, it will be a bit
	 slower to compute the next mark but the algorithms will all behave
	 correctly. We reserve the value '0' to mean 'unmarked'. Thus, this
	 generator starts at '1' and will never produce '0' as a mark value."

	^currentMark := currentMark + 1! !

!DBPlanner methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:36'!
propagateFrom: v
	"The given variable has changed. Propagate new values downstream."

	| todo c |
	todo := OrderedCollection new.
	self addConstraintsConsuming: v to: todo.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 c execute.
		 self addConstraintsConsuming: c output to: todo].! !

!DBPlanner methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:05'!
removePropagateFrom: out
	"Update the walkabout strengths and stay flags of all variables
	 downstream of the given constraint. Answer a collection of unsatisfied
	 constraints sorted in order of decreasing strength."

	| unsatisfied todo v |
	unsatisfied := OrderedCollection new.
	
	out determinedBy: nil.
	out walkStrength: DBStrength absoluteWeakest.
	out stay: true.
	todo := OrderedCollection with: out.
	[todo isEmpty] whileFalse:
		[v := todo removeFirst.
		 v constraints do:
		 	[ :c | (c isSatisfied) ifFalse: [unsatisfied add: c]].
		 self constraintsConsuming: v do:
			[ :c |
			 c recalculate.
			 todo add: c output]].
	
	unsatisfied sort: [ :c1 :c2 | c1 strength stronger: c2 strength].
	^ unsatisfied! !

Object subclass: #DBStrength
	instanceVariableNames: 'symbolicValue arithmeticValue'
	classVariableNames: 'AbsoluteStrongest AbsoluteWeakest Required StrengthConstants StrengthTable'
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBStrength commentStamp: '<historical>' prior: 0!
Strengths are used to measure the relative importance of constraints. The hierarchy of available strengths is determined by the class variable StrengthTable (see my class initialization method). Because Strengths are invariant, references to Strength instances are shared (i.e. all references to
"Strength of: #required" point to a single, shared instance). New strengths may be inserted in the strength hierarchy without disrupting current constraints.

Instance variables:
	symbolicValue		symbolic strength name (e.g. #required) <Symbol>
	arithmeticValue		index of the constraint in the hierarchy, used for comparisons <Number>!

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteStrongest

	^AbsoluteStrongest! !

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteWeakest

	^AbsoluteWeakest! !

!DBStrength class methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:52'!
initialize
	"Initialize the symbolic strength table. Fix the internally caches
	 values of all existing instances."
	"Strength initialize"

	StrengthTable := Dictionary new.
	StrengthTable at: #absoluteStrongest put: -10000.
	StrengthTable at: #required put: -800.
	StrengthTable at: #strongPreferred put: -600.
	StrengthTable at: #preferred put: -400.
	StrengthTable at: #strongDefault put: -200.
	StrengthTable at: #default put: 0.
	StrengthTable at: #weakDefault put: 500.
	StrengthTable at: #absoluteWeakest put: 10000.

	StrengthConstants := Dictionary new.
	StrengthTable keys do:
		[: strengthSymbol |
			StrengthConstants
				at: strengthSymbol
				put: ((super new) initializeWith: strengthSymbol)].

	AbsoluteStrongest := DBStrength of: #absoluteStrongest.
	AbsoluteWeakest := DBStrength of: #absoluteWeakest.
	Required := DBStrength of: #required.! !

!DBStrength class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:52'!
of: aSymbol
	"Answer an instance with the specified strength."

	^ StrengthConstants at: aSymbol! !

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
required

	^Required! !

!DBStrength class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:39'!
strengthTable
	^ StrengthTable! !

!DBStrength methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:50'!
arithmeticValue
	"Answer my arithmetic value. Used for comparisons. Note that
	 STRONGER constraints have SMALLER arithmetic values."

	^arithmeticValue! !

!DBStrength methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:38'!
initializeWith: symVal
	"Record my symbolic value and reset my arithmetic value."

	symbolicValue := symVal.
	arithmeticValue := DBStrength strengthTable at: symbolicValue.! !

!DBStrength methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:50'!
printOn: aStream
	"Append a string which represents my strength onto aStream."

	aStream nextPutAll: '%', symbolicValue, '%'.! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
sameAs: aStrength
	"Answer true if I am the same strength as the given Strength."

	^ arithmeticValue = aStrength arithmeticValue! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
stronger: aStrength
	"Answer true if I am stronger than the given Strength."

	^ arithmeticValue < aStrength arithmeticValue! !

!DBStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
strongest: aStrength
	"Answer the stronger of myself and aStrength."

	(aStrength stronger: self)
		ifTrue: [^aStrength]
		ifFalse: [^self]! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
weaker: aStrength
	"Answer true if I am weaker than the given Strength."

	^ arithmeticValue > aStrength arithmeticValue! !

!DBStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
weakest: aStrength
	"Answer the weaker of myself and aStrength."

	(aStrength weaker: self)
		ifTrue: [^aStrength]
		ifFalse: [^self].! !

Object subclass: #DBVariable
	instanceVariableNames: 'constraints determinedBy mark stay value walkStrength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-DeltaBlue'!

!DBVariable commentStamp: '<historical>' prior: 0!
I represent a constrained variable. In addition to my value, I maintain the structure of the constraint graph, the current dataflow graph, and various parameters of interest to the DeltaBlue incremental constraint solver.

Instance variables:
	value			my value; changed by constraints, read by client <Object>
	constraints		normal constraints that reference me <Array of Constraint>
	determinedBy	the constraint that currently determines
					my value (or nil if there isn''t one) <Constraint>
	walkStrength		my walkabout strength <Strength>
	stay			true if I am a planning-time constant <Boolean>
	mark			used by the planner to mark constraints <Number>!

!DBVariable class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 11:01'!
value: aValue

	^self new value: aValue! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
addConstraint: aConstraint
	"Add the given constraint to the set of all constraints that refer
	 to me."

	constraints add: aConstraint.! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
constraints
	^ constraints! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy
	^ determinedBy! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy: anObject
	determinedBy := anObject! !

!DBVariable methodsFor: 'initialize' stamp: 'StefanMarr 4/2/2015 16:43'!
initialize
	value := 0.
	constraints := OrderedCollection new: 2.
	determinedBy := nil.
	walkStrength := DBStrength absoluteWeakest.
	stay := true.
	mark := 0.! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
longPrintOn: aStream
	^ super longPrintOn: aStream
"	self shortPrintOn: aStream.
	aStream nextPutAll: '   Constraints: '.
	(constraints isEmpty)
		ifTrue: [aStream cr; tab; nextPutAll: 'none']
		ifFalse:
			[constraints do:
				[: c | aStream cr; tab. c shortPrintOn: aStream]].
	(determinedBy isNil) ifFalse:
		[aStream cr; nextPutAll: '   Determined by: '.
		 aStream cr; tab. determinedBy shortPrintOn: aStream].
	aStream cr."! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark
	^ mark! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark: anObject
	mark := anObject! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 20:59'!
printOn: aStream
	^ super printOn: aStream
	"self shortPrintOn: aStream"! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
removeConstraint: c
	"Remove all traces of c from this variable."

	constraints remove: c ifAbsent: [].
	(determinedBy == c) ifTrue: [determinedBy := nil].! !

!DBVariable methodsFor: 'update' stamp: 'ClementBera 9/23/2013 10:59'!
setValue: aValue
	"Attempt to assign the given value to me using a strength of
	 #preferred."

	self setValue: aValue strength: #preferred.! !

!DBVariable methodsFor: 'update' stamp: 'ClementBera 9/23/2013 11:00'!
setValue: aValue strength: strengthSymbol
	"Attempt to assign the given value to me using the given strength."

	| editConstraint |
	editConstraint := DBEditConstraint var: self strength: strengthSymbol.
	(editConstraint isSatisfied) ifTrue:
		[self value: aValue.
		 DBPlanner propagateFrom: self].
	editConstraint destroyConstraint.! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
shortPrintOn: aStream
	^ super shortPrintOn: aStream.
"
	aStream nextPutAll: 'V(', self printString, ', '.
	aStream nextPutAll: walkStrength printString, ', '.
	(stay isNil) ifFalse:
		[aStream nextPutAll: (stay ifTrue: ['stay, '] ifFalse: ['changing, '])].
	aStream nextPutAll: value printString.
	aStream nextPutAll: ')'.
	aStream cr."! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
stay
	^ stay! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
stay: aBoolean
	stay := aBoolean! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value
	^ value! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value: anObject
	value := anObject! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
walkStrength
	^ walkStrength! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
walkStrength: aStrength
	walkStrength := aStrength! !

Object subclass: #GSEdge
	instanceVariableNames: 'dest weight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-GraphSearch'!

!GSEdge class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:25'!
newWith: dest and: weight
	^ self basicNew initializeWith: dest and: weight! !

!GSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:24'!
dest
	^ dest! !

!GSEdge methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:25'!
initializeWith: destination and: w
	dest   := destination.
	weight := w.! !

!GSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:25'!
weight
	^ weight! !

Object subclass: #GSNode
	instanceVariableNames: 'starting noOfEdges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-GraphSearch'!

!GSNode class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:24'!
newWith: starting and: noOfEdges
	^ self basicNew initializeWith: starting and: noOfEdges! !

!GSNode methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:23'!
initializeWith: start and: edges
	starting := start.
	noOfEdges:= edges! !

!GSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
noOfEdges
	^ noOfEdges! !

!GSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
starting
	^ starting! !

Object subclass: #JSHashIndexTable
	instanceVariableNames: 'hashTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:22'!
at: name
    | slot |
    slot := self hashSlotFor: name.

    " subtract 1, 0 stands for empty "
    ^ ((hashTable at: slot) & 255) - 1! !

!JSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:16'!
at: name put: index
    | slot |
    slot := self hashSlotFor: name.
    
    index < 255
      ifTrue:  [ hashTable at: slot put: index + 1 ]
      ifFalse: [ hashTable at: slot put: 0 ]! !

!JSHashIndexTable methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 13:06'!
hashSlotFor: element
    ^ (element hash & 31) + 1! !

!JSHashIndexTable methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:16'!
initialize
	hashTable := IntegerArray new: 32 withAll: 0! !

!JSHashIndexTable methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:21'!
remove: index
    1 to: hashTable size do: [:i |
      index + 1 = (hashTable at: i)
        ifTrue:  [ hashTable at: i put: 0 ]
        ifFalse: [ 
          (hashTable at: i) > index + 1 ifTrue: [
            hashTable at: i put: (hashTable at: i) - 1]]]! !

Object subclass: #JSJsonParser
	instanceVariableNames: 'input index line column current captureBuffer captureStart exceptionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonParser class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 11:38'!
with: aJsonString
    ^ self basicNew initializeWith: aJsonString! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:03'!
endCapture
    | captured |
    '' = captureBuffer
      ifTrue:  [ captured := input copyFrom: captureStart to: index - 1 ]
      ifFalse: [
        self pauseCapture.
        captured := captureBuffer.
        captureBuffer := '' ].
    captureStart := -1.
    
    ^ captured! !

!JSJsonParser methodsFor: 'error handling' stamp: 'StefanMarr 4/3/2015 11:37'!
error: message
    'error:' print.
    message print.
    ':' print.
    line print.
    column println.
    exceptionBlock value: (JSParseException with: message at: index
                                          line: line    column: column )
  ! !

!JSJsonParser methodsFor: 'error handling' stamp: 'StefanMarr 4/3/2015 11:45'!
expected: expected
    self isEndOfText ifTrue: [
      self error: 'Unexpected end of input, expected ', expected asString
    ].
    self error: 'Expected ', expected! !

!JSJsonParser methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:31'!
initializeWith: string
    input := string.
    index := 0.
    line  := 1.
    column := 0.
    current := nil.
    captureBuffer := ''.! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isDigit
    current = '0' ifTrue: [^ true].
    current = '1' ifTrue: [^ true].
    current = '2' ifTrue: [^ true].
    current = '3' ifTrue: [^ true].
    current = '4' ifTrue: [^ true].
    current = '5' ifTrue: [^ true].
    current = '6' ifTrue: [^ true].
    current = '7' ifTrue: [^ true].
    current = '8' ifTrue: [^ true].
    current = '9' ifTrue: [^ true].
    ^ false! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isEndOfText
    ^ current isNil! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isWhiteSpace
    current = ' '  ifTrue: [^ true].
    current = '\t' ifTrue: [^ true].
    current = '\n' ifTrue: [^ true].
    current = '\r' ifTrue: [^ true].
    ^ false! !

!JSJsonParser methodsFor: 'parsing' stamp: 'StefanMarr 4/3/2015 11:31'!
parse
    | result |
    exceptionBlock := [:ex | ^ ex ].
    self read.
    self skipWhiteSpace.
    result := self readValue.
    self skipWhiteSpace.
    self isEndOfText ifFalse: [ self error: 'Unexpected character'].
    ^ result! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:37'!
pauseCapture
    captureBuffer := captureBuffer concatenate: (
      input substringFrom: captureStart to: index - 1).
    captureStart := -1! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:02'!
read
	"TODO: this is probably broken"
    current = '\n' ifTrue: [
      line := line + 1.
      column := 0.
    ].
    
    index := index + 1.
    column := column + 1.

    input ifNil: [ self error:'input nil'].
    index <= input size
      ifTrue:  [ current := input copyFrom: index to: index ]
      ifFalse: [ current := nil ]! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readArray
    | array |
    self read.
    array := JSJsonArray new.
    
    "Array might be empty"
    self skipWhiteSpace.
    (self readChar: ']') ifTrue: [
      ^ array
    ].
    
    self readArrayElement: array.
    [self readChar: ','] whileTrue: [
      self readArrayElement: array.
    ].

    (self readChar: ']') ifFalse: [
      self expected: '"," or "]"'
    ].
    ^ array! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readArrayElement: array
    self skipWhiteSpace.
    array add: self readValue.
    self skipWhiteSpace.! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readChar: ch
    current = ch ifFalse: [ ^ false ].
    self read.
    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readDigit
    self isDigit ifFalse: [ ^ false ].
    self read.
    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscape
    self read.    
    captureBuffer := captureBuffer concatenate: self readEscapeChar.
    self read! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscapeChar
    current = '"'  ifTrue: [ ^ '"'  ].
    current = '/'  ifTrue: [ ^ '/'  ].
    current = '\\' ifTrue: [ ^ '\\' ].

    current = 'b'  ifTrue: [ ^ '\b'  ].
    current = 'f'  ifTrue: [ ^ '\f'  ].
    current = 'n'  ifTrue: [ ^ '\n'  ].
    current = 'r'  ifTrue: [ ^ '\r'  ].
    current = 't'  ifTrue: [ ^ '\t'  ].

    "TODO: SOM doesn't have a way to create unicode characters."
    self expected: 'valid escape sequence. note, some are not supported'
! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readExponent
    ((self readChar: 'e') not and: [
      (self readChar: 'E') not]) ifTrue: [ ^ false ].
    
    
    (self readChar: '+') ifFalse: [ self readChar: '-' ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].

    [self readDigit] whileTrue: [].

    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readFalse
    self read.
    self readRequiredChar: 'a'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 's'.
    self readRequiredChar: 'e'.
    ^ JSJsonLiteral FALSE! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readFraction
    (self readChar: '.') ifFalse: [ ^ false ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    
    [self readDigit] whileTrue: [].

    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readName
    current = '"' ifFalse: [ self expected: 'name' ].
    ^ self readStringInternal! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readNull
    self read.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 'l'.
    ^ JSJsonLiteral NULL! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:05'!
readNumber
    | firstDigit |
    self startCapture.
    self readChar: '-'.
    firstDigit := current.
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    firstDigit ~= '0' ifTrue: [ [self readDigit] whileTrue: []].
        
    self readFraction.
    self readExponent.
    ^ JSJsonNumber new: self endCapture! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readObject
    | object |
    self read.
    object := JSJsonObject new.
    self skipWhiteSpace.

    (self readChar: '}') ifTrue: [
      ^ object
    ].
    
    self readObjectKeyValuePair: object.
    [self readChar: ','] whileTrue: [
      self readObjectKeyValuePair: object.
    ].
    
    (self readChar: '}') ifFalse: [
      self expected: '"," or "}"'
    ].
    
    ^ object! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readObjectKeyValuePair: object
    | name |
    self skipWhiteSpace.
    name := self readName.
    self skipWhiteSpace.
  
    (self readChar: ':') ifFalse: [ self expected: ':' ].
  
    self skipWhiteSpace.
  
    object add: name with: self readValue.
  
    self skipWhiteSpace! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readRequiredChar: ch
    (self readChar: ch) ifFalse: [
      self expected: 'character: ' + ch
    ]! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readString
    ^ JSJsonString new: self readStringInternal! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readStringInternal
    | string |
    self read.
    self startCapture.
    
    [current = '"'] whileFalse: [
      current = '\\' ifTrue: [
        self pauseCapture.
        self readEscape.
        self startCapture.
      ] ifFalse: [
        "if (current < 0x20) { throw expected('valid string character'); }"
        "we currently don't have a way to get the ordinal value for a character"
        "} else {"
        self read.
      ]
    ].
    string := self endCapture.
    self read.
    ^ string! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readTrue
    self read.
    self readRequiredChar: 'r'.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'e'.
    ^ JSJsonLiteral TRUE! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:31'!
readValue
    current = 'n' ifTrue: [ ^ self readNull   ].
    current = 't' ifTrue: [ ^ self readTrue   ].
    current = 'f' ifTrue: [ ^ self readFalse  ].
    current = '"' ifTrue: [ ^ self readString ].
    current = '[' ifTrue: [ ^ self readArray  ].
    current = '{' ifTrue: [ ^ self readObject ].
    
    "Is this really the best way to write this?, or better #or:?,
     but with all the nesting, it's just ugly."
    current = '-' ifTrue: [ ^ self readNumber ].
    current = '0' ifTrue: [ ^ self readNumber ].
    current = '1' ifTrue: [ ^ self readNumber ].
    current = '2' ifTrue: [ ^ self readNumber ].
    current = '3' ifTrue: [ ^ self readNumber ].
    current = '4' ifTrue: [ ^ self readNumber ].
    current = '5' ifTrue: [ ^ self readNumber ].
    current = '6' ifTrue: [ ^ self readNumber ].
    current = '7' ifTrue: [ ^ self readNumber ].
    current = '8' ifTrue: [ ^ self readNumber ].
    current = '9' ifTrue: [ ^ self readNumber ].
    
    "else"
    self expected: 'value'! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
skipWhiteSpace
    [ self isWhiteSpace ]
      whileTrue:
        [ self read ].! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
startCapture
    captureStart := index! !

Object subclass: #JSJsonValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

JSJsonValue subclass: #JSJsonArray
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonArray class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:52'!
readFrom: string
    ^ (JSJsonValue readFrom: string) asArray! !

!JSJsonArray methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:51'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ values = other values! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:49'!
add: value
    value ifNil: [ self error: 'value is null' ].
    values add: value! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addBoolean: value
    values add: (JSJsonValue boolean: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addDouble: value
	values add: (JSJsonValue double: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addInteger: value
    values add: (JSJsonValue integer: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:49'!
addString: value
    values add: (JSJsonValue string: value)! !

!JSJsonArray methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:51'!
asArray
	^ self! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index
	^ values at: index! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index put: value
    value ifNil: [ self error: 'value is null' ].
    values at: index put: value! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putBoolean: value
    values at: index put: (JSJsonValue boolean: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putDouble: value
    values at: index put: (JSJsonValue double: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putInteger: value
    values at: index put: (JSJsonValue integer: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index putString: value
    values at: index put: (JSJsonValue string: value)! !

!JSJsonArray methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:51'!
hashcode
	^ values hash! !

!JSJsonArray methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:47'!
initialize
	values := OrderedCollection new.! !

!JSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:51'!
isArray
	^ true! !

!JSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:50'!
isEmpty
    ^ values isEmpty! !

!JSJsonArray methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:50'!
removeAt: index
    values removeAt: index! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
size
    ^ values size! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:51'!
values
	^ values! !

JSJsonValue subclass: #JSJsonLiteral
	instanceVariableNames: 'value isNull isTrue isFalse'
	classVariableNames: 'FALSE NULL TRUE'
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
FALSE
	^ FALSE! !

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
NULL
	^ NULL! !

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
TRUE
	^ TRUE! !

!JSJsonLiteral class methodsFor: 'class initialization' stamp: 'StefanMarr 4/3/2015 13:13'!
initialize
   "self initialize"
	NULL := self basicNew initializeWith: 'null'.
	TRUE := self basicNew initializeWith: 'true'.
	FALSE:= self basicNew initializeWith: 'false'.! !

!JSJsonLiteral methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:50'!
= other
    self == other ifTrue: [ ^ true ].  
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ value = other asString
! !

!JSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:49'!
asBoolean
	isNull ifTrue: [ ^ super asBoolean ]
	       ifFalse:[ ^ isTrue ]! !

!JSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:44'!
asString
	^ value! !

!JSJsonLiteral methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:44'!
hashcode
	^ value hash! !

!JSJsonLiteral methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:44'!
initializeWith: val
    value   := val.
    isNull  := 'null'  = val.
    isTrue  := 'true'  = val.
    isFalse := 'false' = val.! !

!JSJsonLiteral methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:49'!
isBoolean
	^ isTrue | isFalse! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isFalse
	^ isFalse! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isNull
	^ isNull! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isTrue
	^ isTrue! !

JSJsonValue subclass: #JSJsonNumber
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonNumber class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 11:05'!
new: string
    string ifNil: [ self error: 'string is null' ].
    ^ self new initializeWith: string
! !

!JSJsonNumber methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:58'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    
    self class == other class ifFalse: [ ^ false ].
    ^ string = other asString
! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:57'!
asDouble
	^ Float readFromString: string! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:53'!
asInteger
	^ string asInteger! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:52'!
asString
	^ string! !

!JSJsonNumber methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:58'!
hashcode
	^ string hash! !

!JSJsonNumber methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:52'!
initializeWith: str
	string := str! !

!JSJsonNumber methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:52'!
isNumber
	^ true! !

JSJsonValue subclass: #JSJsonObject
	instanceVariableNames: 'names values table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonObject class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 11:26'!
readFrom: string
	^ (JSJsonValue readFrom: string) asObject! !

!JSJsonObject methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 11:16'!
= other
    self == other ifTrue: [ ^ true  ].
    self == nil   ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].
    
    ^ names = other names & values = other values! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 13:05'!
add: name with: aJsonValue
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].

    names  add: name.
    values add: aJsonValue.
    table at: name put: names size.! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withBoolean: value
    self add: name with: (JSJsonValue boolean: value)! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withNumber: value
    self add: name with: (JSJsonValue number: value)! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withString: value
    self add: name with: (JSJsonValue string: value)! !

!JSJsonObject methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:15'!
asObject
	^ self! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    idx := self indexOf: name.
    idx = -1
      ifTrue:  [ ^ nil ]
      ifFalse: [ ^ values at: idx ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asBooleanWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asBoolean ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asDoubleWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asDouble ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asIntegerWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asInteger ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asStringWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asString ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:12'!
at: name put: aJsonValue
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].
  
    idx := self indexOf: name.
    idx <> -1
      ifTrue:  [values at: idx put: aJsonValue]
      ifFalse: [
        names  append: name.
        values append: aJsonValue.
        table at: name put: names size.
      ].! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putBoolean: value
    self at: name put: (JSJsonValue boolean: value)! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putNumber: value
    self at: name put: (JSJsonValue number: value)! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putString: value
    self at: name put: (JSJsonValue string: value)! !

!JSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:15'!
hashcode
	| result |
    result := 1.
    result := 31 * result + names hash.
    result := 31 * result + values hash.
    ^ result
! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:15'!
indexOf: name
    | idx |
    idx := table at: name.    
    idx ~= -1 & name = (names at: idx) ifTrue: [ ^ idx ].
    ^ names lastIndexOf: name! !

!JSJsonObject methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:08'!
initialize
    names  := OrderedCollection new.
    values := OrderedCollection new.
    table  := JSHashIndexTable new
! !

!JSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isEmpty
	^ names isEmpty! !

!JSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isObject
	^ true! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
names
	^ names! !

!JSJsonObject methodsFor: 'removing' stamp: 'StefanMarr 4/3/2015 11:14'!
remove: name
    | idx |
    name ifNil: [ self error: 'name is null' ].
    idx := self indexOf: name.
    idx = -1 ifFalse: [
      table  remove: name.
      names  remove: idx.
      values remove: idx.
    ].! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
size
	^ names size! !

!JSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:17'!
updateHashIndex
    | size |
    size := names size.
    1 to: size do: [: i |
      table add: (names at: i) put: i
    ]! !

JSJsonValue subclass: #JSJsonString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSJsonString class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:33'!
new: str
	^ self basicNew initializeWith: str! !

!JSJsonString methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:35'!
= other
	self == other ifTrue: [ ^ true ].
	other == nil ifTrue: [ ^ false ].

	self class == other class ifFalse: [ ^ false ].

	^ string =  other asString! !

!JSJsonString methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:34'!
asString
	^ string! !

!JSJsonString methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:34'!
hashcode
	^ string hash! !

!JSJsonString methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:34'!
initializeWith: str
	string := str! !

!JSJsonString methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:34'!
isString
	^ true! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:31'!
boolean: aBoolean
	^ aBoolean
		ifTrue:  [ JSJsonLiteral TRUE ]
		ifFalse: [ JSJsonLiteral FALSE ]! !

!JSJsonValue class methodsFor: 'helper' stamp: 'StefanMarr 4/2/2015 22:32'!
cutOffPointZero: str
    (str endsWith: '.0') ifTrue: [ ^ str substringFrom: 1 to: str length - 2].
    ^ str! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:30'!
double: aDouble
	aDouble ifNil: [ self error: 'aDouble nil' ].
	^ JSJsonNumber new: (self cutOffPointZero: aDouble asString)! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:28'!
integer: anInteger
	^ JSJsonNumber new: anInteger asString! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:28'!
readFrom: str 
	^ (JSJsonParser with: str) parse! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:31'!
string: aString
	^ aString == nil
		ifTrue:  [ JSJsonLiteral NULL ]
		ifFalse: [ JSJsonString new: aString ]! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asArray
	self error: 'Unsupported operation, not an array ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asBoolean
	self error: 'Unsupported operation, not a boolean ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asDouble
	self error: 'Unsupported operation, not a double ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asInteger
	self error: 'Unsupported operation, not an integer ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asObject
	self error: 'Unsupported operation, not an object: ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asString
	self error: 'Unsupported operation, not a string ' + self asString! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isArray
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isBoolean
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isFalse
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isNull
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isNumber
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isObject
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isString
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isTrue
	^ false! !

!JSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeOn: writer
	^ self subclassResponsibility! !

!JSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeTo: writer
	self writeTo: writer with: nil! !

Object subclass: #JSParseException
	instanceVariableNames: 'msg offset line column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!JSParseException methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:41'!
asString ^ msg, ':', line, ':', column! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
column ^ column! !

!JSParseException methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:40'!
initializeWith: message at: anOffset line: aLine column: aColumn
    msg    := message.
    offset := anOffset.
    line   := aLine.
    column := aColumn.! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
line ^ line! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
message ^ msg! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
offset ^ offset! !

Object subclass: #NBBody
	instanceVariableNames: 'x y z vx vy vz mass'
	classVariableNames: 'solarMass'
	poolDictionaries: ''
	category: 'SOM-Benchmarks-NBody'!

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
DaysPerYear
	^ 365.24! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
Pi
	^ 3.141592653589793! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
SolarMass
	^ solarMass! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
initialize
	"self initialize"
	solarMass := 4 * self Pi * self Pi.
	! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
jupiter
		| b |
       b := super new.
       b x:    4.8414314424647209.
       b y:   -1.16032004402742839.
       b z:   -0.103622044471123109.
       b vx:   0.00166007664274403694   * self DaysPerYear.
       b vy:   0.00769901118419740425   * self DaysPerYear.
       b vz:  -0.0000690460016972063023 * self DaysPerYear.
       b mass: 0.000954791938424326609  * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
neptune
		| b |
       b := super new.
       b x:   15.3796971148509165.
       b y:  -25.9193146099879641.
       b z:    0.179258772950371181.
       b vx:   0.00268067772490389322   * self DaysPerYear.
       b vy:   0.00162824170038242295   * self DaysPerYear.
       b vz:  -0.000095159225451971587  * self DaysPerYear.
       b mass: 0.0000515138902046611451 * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
new
	| b |
        b := super new.
        b x: 0.0.  b vx: 0.0.
        b y: 0.0.  b vy: 0.0.
        b z: 0.0.  b vz: 0.0.
        b mass: 0.0.
        ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
saturn 
		| b |
       b := super new.
       b x:    8.34336671824457987.
       b y:    4.12479856412430479.
       b z:   -0.403523417114321381.
       b vx:  -0.00276742510726862411   * self DaysPerYear.
       b vy:   0.00499852801234917238   * self DaysPerYear.
       b vz:   0.0000230417297573763929 * self DaysPerYear.
       b mass: 0.000285885980666130812  * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
sun 
		| b |
       b := self new.
       b mass: self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
uranus
		| b |
       b := super new.
       b x:   12.894369562139131.
       b y:  -15.1111514016986312.
       b z:   -0.223307578892655734.
       b vx:   0.00296460137564761618   * self DaysPerYear.
       b vy:   0.0023784717395948095    * self DaysPerYear.
       b vz:  -0.0000296589568540237556 * self DaysPerYear.
       b mass: 0.0000436624404335156298 * self SolarMass.
       ^ b! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
mass ^ mass! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
mass: val
	mass := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
offsetMomentumX: px y: py z: pz
       vx := 0.0 - (px / NBBody SolarMass).
       vy := 0.0 - (py / NBBody SolarMass).
       vz := 0.0 - (pz / NBBody SolarMass).! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
vx ^ vx! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vx: val
	vx := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy ^ vy! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy: val
	vy := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vz ^ vz! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
vz: val
	vz := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
x ^ x! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
x: val
	x := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
y ^ y! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
y: val
	y := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
z ^ z! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
z: val
	z := val! !

Object subclass: #NBNBodySystem
	instanceVariableNames: 'bodies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-NBody'!

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
advance: dt
        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.

            i + 1 to: bodies size do: [:j |
                | dx dy dz jBody dSquared distance mag |
                jBody := bodies at: j.
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                dSquared := (dx * dx) + (dy * dy) + (dz * dz).
                distance := dSquared sqrt.
                mag      := dt / (dSquared * distance).

                iBody vx: iBody vx - (dx * jBody mass * mag).
                iBody vy: iBody vy - (dy * jBody mass * mag).
                iBody vz: iBody vz - (dz * jBody mass * mag).

                jBody vx: jBody vx + (dx * iBody mass * mag).
                jBody vy: jBody vy + (dy * iBody mass * mag).
                jBody vz: jBody vz + (dz * iBody mass * mag).
            ].
       ].
       
       bodies do: [:body |
           body x: body x + (dt * body vx).
           body y: body y + (dt * body vy).
           body z: body z + (dt * body vz).
       ].! !

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
energy
        | dx dy dz distance e |
        e := 0.0.

        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.
            
            e := e + (0.5 * iBody mass *
                 ((iBody vx * iBody vx) +
                  (iBody vy * iBody vy) +
                  (iBody vz * iBody vz))).
            
            i + 1 to: bodies size do: [:j |
                | jBody |
                jBody := bodies at: j.
                
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
                e := e - ((iBody mass * jBody mass) / distance).
            ].
       ].
       ^ e! !

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:14'!
initialize
        | px py pz |

        bodies := Array new: 5.
        bodies at: 1 put: NBBody sun.
        bodies at: 2 put: NBBody jupiter.
        bodies at: 3 put: NBBody saturn.
        bodies at: 4 put: NBBody uranus.
        bodies at: 5 put: NBBody neptune.
        
        "bodies do: [:b | b print. '' println ]."

        px := 0.0.  py := 0.0.  pz := 0.0.

        bodies do: [:b |
            px := px + (b vx * b mass).
            py := py + (b vy * b mass).
            pz := pz + (b vz * b mass).
        ].
        
        (bodies at: 1) offsetMomentumX: px y: py z: pz.
        
        "bodies do: [:b | b print. '' println ]."! !

Object subclass: #RichObject
	instanceVariableNames: ''
	classVariableNames: 'DeviceA DeviceB DevicePacketKind HandlerA HandlerB Idler NoTask NoWork WorkPacketKind Worker'
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichObject commentStamp: '<historical>' prior: 0!
This class mostly adds some constants that are used in the Richards benchmarks.!

RichObject subclass: #RichDeviceTaskDataRecord
	instanceVariableNames: 'pending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichDeviceTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that suspends itself after each time it has been run to simulate waiting for data from an external device.
!

!RichDeviceTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:40'!
create
	^ super basicNew create! !

!RichDeviceTaskDataRecord methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 13:39'!
create
	pending := RichObject NoWork! !

!RichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
pending
	^ pending! !

!RichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:39'!
pending: packet
	pending := packet! !

RichObject subclass: #RichHandlerTaskDataRecord
	instanceVariableNames: 'workIn deviceIn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichHandlerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets and then suspends itself.!

!RichHandlerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:03'!
create
	^ super basicNew create! !

!RichHandlerTaskDataRecord methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 13:42'!
asString
	^ 'HandlerTaskDataRecord(', workIn asString, ', ', deviceIn asString, ')'! !

!RichHandlerTaskDataRecord methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:42'!
create
	workIn := deviceIn := RichObject NoWork! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceIn
	^ deviceIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
deviceIn: aPacket
	deviceIn := aPacket! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceInAdd: packet
    deviceIn := self append: packet head: deviceIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
workIn
	^ workIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
workIn: aWorkQueue
	workIn := aWorkQueue! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
workInAdd: packet
    workIn := self append: packet head: workIn! !

RichObject subclass: #RichIdleTaskDataRecord
	instanceVariableNames: 'control count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichIdleTaskDataRecord commentStamp: '<historical>' prior: 0!
An idle task doesn't do any work itself but cycles control between the two device tasks.!

!RichIdleTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:43'!
create
	^ super basicNew create! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
control
	^ control! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
control: aNumber
	control := aNumber! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
count
	^ count! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
count: aCount
	count := aCount! !

!RichIdleTaskDataRecord methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:04'!
create
    control := 1.
    count := 10000! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceA
	^ DeviceA! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceB
	^ DeviceB! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DevicePacketKind
	^ DevicePacketKind! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerA
	^ HandlerA! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerB
	^ HandlerB! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
Idler
	^ Idler! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoTask
	^ NoTask! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoWork
	^ NoWork! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
WorkPacketKind
	^ WorkPacketKind! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
Worker
	^ Worker! !

!RichObject class methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 13:37'!
initialize
    	DeviceA := 5.
    	DeviceB := 6.
    	DevicePacketKind := 1.
    	HandlerA := 3.
    	HandlerB := 4.
    	Idler := 1.
	   NoWork := nil.
	   NoTask := nil.
    	Worker := 2.
    	WorkPacketKind := 2! !

!RichObject methodsFor: 'utilities' stamp: 'StefanMarr 4/3/2015 13:35'!
append: packet head: queueHead
    | mouse link |
    packet link: RichObject NoWork.
    RichObject NoWork == queueHead ifTrue: [ ^ packet ].
    mouse := queueHead.
    [RichObject NoWork == (link := mouse link)]
        whileFalse: [ mouse := link ].
    mouse link: packet.
    ^ queueHead! !

RichObject subclass: #RichPacket
	instanceVariableNames: 'data datum identity kind link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichPacket commentStamp: '<historical>' prior: 0!
A simple package of data that is manipulated by the tasks.  The exact layout of the payload data carried by a packet is not importaint, and neither is the nature of the work performed on packets by the tasks. Besides carrying data, packets form linked lists and are hence used both as data and worklists.!

!RichPacket class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:44'!
create: link identity: identity kind: kind 
    ^ self basicNew
		link: link
		identity: identity
		kind: kind! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
data
	^ data! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum
	^ datum! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum: anObject
	datum := anObject! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity
	^ identity! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity: anObject
	identity := anObject! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
kind
	^ kind! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
link
	^ link! !

!RichPacket methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:45'!
link: aWorkQueue
	link := aWorkQueue! !

!RichPacket methodsFor: 'initialize' stamp: 'ClementBera 9/27/2013 15:20'!
link: aLink identity: anIdentity kind: aKind 
    link := aLink.
    identity := anIdentity. 
    kind := aKind.
    datum := 1.
    data := ByteArray new: 4! !

RichObject subclass: #RichTaskState
	instanceVariableNames: 'packetPending taskHolding taskWaiting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichTaskState commentStamp: '<historical>' prior: 0!
Abstract task that manipulates work packets.!

RichTaskState subclass: #RichTaskControlBlock
	instanceVariableNames: 'link identity priority input function handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichTaskControlBlock commentStamp: '<historical>' prior: 0!
A task control block manages a task and the queue of work packages associated with it.!

!RichTaskControlBlock class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:17'!
link: link create: identity priority: priority initialWorkQueue: initialWorkQueue initialState: initialState function: aBlock privateData: privateData 
    ^ self new
		link: link
		identity: identity
		priority: priority
		initialWorkQueue: initialWorkQueue
		initialState: initialState
		function: aBlock
		privateData: privateData! !

!RichTaskControlBlock methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:35'!
addInput: packet checkPriority: oldTask
    RichObject NoWork == input
	ifTrue: 
	    [input := packet.
	    self packetPending: true.
	    priority > oldTask priority ifTrue: [ ^ self ]]
	ifFalse: 
	    [ input := self append: packet head: input ].
    ^ oldTask! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
identity
	^ identity! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
link
	^ link! !

!RichTaskControlBlock methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:16'!
link: aLink identity: anIdentity priority: aPriority initialWorkQueue: anInitialWorkQueue initialState: anInitialState function: aBlock privateData: aPrivateData 
    link := aLink.
    identity := anIdentity.
    priority := aPriority.
    input := anInitialWorkQueue.
    self packetPending: anInitialState isPacketPending.
    self taskWaiting: anInitialState isTaskWaiting.
    self taskHolding: anInitialState isTaskHolding.
    function := aBlock.
    handle := aPrivateData! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
priority
	^ priority! !

!RichTaskControlBlock methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:35'!
runTask
    | message |
    self isWaitingWithPacket
	ifTrue: 
	    [message := input.
	    input := message link.
	    RichObject NoWork == input 
		ifTrue: [self running]
		ifFalse: [self packetPending]]
	ifFalse: [message := RichObject NoWork].
    ^ function value: message value: handle! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
packetPending
    ^super new packetPending! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
running
    ^super new running! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waiting
    ^super new waiting! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waitingWithPacket
    ^super new waitingWithPacket! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:36'!
isPacketPending
    ^packetPending! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isRunning
    ^ packetPending not and: [taskWaiting not and: [taskHolding not]]! !

!RichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskHolding
    ^taskHolding! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isTaskHoldingOrWaiting
    ^taskHolding or: [packetPending not and: [taskWaiting]]! !

!RichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskWaiting
    ^taskWaiting! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaiting
    ^packetPending not and: [taskWaiting and: [taskHolding not]]! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaitingWithPacket
    ^packetPending and: [taskWaiting and: [taskHolding not]]! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending
    packetPending := true.
    taskWaiting := false.
    taskHolding := false! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending: aBoolean
	packetPending := aBoolean! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
running
    packetPending := taskWaiting := taskHolding := false! !

!RichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskHolding: aBoolean
	^ taskHolding := aBoolean! !

!RichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskWaiting: aBoolean
	^ taskWaiting := aBoolean! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waiting
    packetPending := taskHolding := false.
    taskWaiting := true! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waitingWithPacket
    taskHolding := false.
    taskWaiting := packetPending := true! !

RichObject subclass: #RichWorkerTaskDataRecord
	instanceVariableNames: 'destination count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichWorkerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets.!

!RichWorkerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 14:40'!
create
	^ super basicNew create! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
count
	^ count! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:16'!
count: anInteger
	count := anInteger! !

!RichWorkerTaskDataRecord methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 14:41'!
create
    destination := RichObject HandlerA.
    count := 0! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination
	^ destination! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination: anObject
	destination := anObject! !

Object subclass: #RichRunner
	instanceVariableNames: 'taskList currentTask currentTaskIdentity taskTable tracing layout queuePacketCount holdCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!RichRunner commentStamp: '<historical>' prior: 0!
Richards simulates the task dispatcher of an operating system.!

!RichRunner class methodsFor: 'starting' stamp: 'ClementBera 9/27/2013 15:58'!
start
    "RichardsBenchmark start"

    ^self new start! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 13:53'!
createDevice: identity priority: priority work: work state: state 
    | data |
    data := RichDeviceTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 functionWork |
	    data1 := word.
	    functionWork := work1.
	    RichObject NoWork == functionWork 
		ifTrue:
		    [RichObject NoWork == (functionWork := data1 pending)
			ifTrue: [self wait]
			ifFalse: 
			    [data1 pending: RichObject NoWork.
			    self queuePacket: functionWork]]
		ifFalse: 
		    [data1 pending: functionWork.
			tracing ifTrue: [ self trace: functionWork datum ].
		    self holdSelf]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:15'!
createHandler: identity priority: priority work: work state: state 
    | data |
    data := RichHandlerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 workPacket |
	    data1 := word.
	    RichObject NoWork == work1 
		ifFalse: [RichObject WorkPacketKind == work1 kind 
		    ifTrue: [data1 workInAdd: work1]
		    ifFalse: [data1 deviceInAdd: work1]].
	    RichObject NoWork == (workPacket := data1 workIn) 
		ifTrue: [self wait]
		ifFalse: 
		    [ | count |
			count := workPacket datum.
		    count > 4
			ifTrue: 
			    [data1 workIn: workPacket link.
			    self queuePacket: workPacket]
			ifFalse:
			    [ | devicePacket |
				RichObject NoWork == (devicePacket := data1 deviceIn) 
				ifTrue: [self wait]
				ifFalse: 
				    [data1 deviceIn: devicePacket link.
				    devicePacket datum: (workPacket data at: count).
				    workPacket datum: count + 1.
				    self queuePacket: devicePacket]]]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:14'!
createIdler: identity priority: priority work: work state: state 
    | data |
    data := RichIdleTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    data1 count: data1 count - 1.
	    0 = data1 count
		ifTrue: [self holdSelf]
		ifFalse:
		    [0 = (data1 control bitAnd: 1)
			ifTrue: 
			    [data1 control: data1 control // 2.
			    self release: RichObject DeviceA]
			ifFalse: 
			    [data1 control: (data1 control // 2 bitXor: 53256).
			    self release: RichObject DeviceB]]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'ClementBera 9/27/2013 15:27'!
createPacket: link identity: identity kind: kind 
    ^ RichPacket
		create: link
		identity: identity
		kind: kind! !

!RichRunner methodsFor: 'creation' stamp: 'ClementBera 9/27/2013 15:28'!
createTask: identity priority: priority work: work state: state function: aBlock data: data 
    | t |
    t := RichTaskControlBlock
		link: taskList
		create: identity
		priority: priority
		initialWorkQueue: work
		initialState: state
		function: aBlock
		privateData: data.
    taskList := t.
    taskTable at: identity put: t! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:15'!
createWorker: identity priority: priority work: work state: state 
    | data |
    data := RichWorkerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    RichObject NoWork == work1
		ifTrue: [self wait]
		ifFalse: 
		    [data1 destination: (RichObject HandlerA = data1 destination
			    ifTrue:  [RichObject HandlerB]
			    ifFalse: [RichObject HandlerA]).
		    work1 identity: data1 destination.
		    work1 datum: 1.
		    1 to: 4 do: [ :i | 
			data1 count: data1 count + 1.
			data1 count > 26 ifTrue: [data1 count: 1].
			work1 data at: i put: $A asInteger + data1 count - 1].
		    self queuePacket: work1]]
	data: data! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:07'!
findTask: identity 
    | t |
    t := taskTable at: identity.
    RichObject NoTask == t ifTrue: [self error: 'findTask failed'].
    ^ t! !

!RichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:35'!
holdSelf
    holdCount := holdCount + 1.
    currentTask taskHolding: true.
    ^ currentTask link! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:08'!
initScheduler
    queuePacketCount := 0.
    holdCount := 0.
    taskTable := Array new: 6 withAll: RichObject NoTask.
    taskList := RichObject NoTask.! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:08'!
initTrace
	tracing := false.
	layout := 0! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:09'!
queuePacket: packet 
    | t |
    t := self findTask: packet identity.
    RichObject NoTask ==  t ifTrue: [ ^ RichObject NoTask ].
    queuePacketCount := queuePacketCount + 1.
    packet link: RichObject NoWork.
    packet identity: currentTaskIdentity.
    ^ t addInput: packet checkPriority: currentTask! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:10'!
release: identity 
    | t |
    t := self findTask: identity.
    RichObject NoTask == t ifTrue: [ ^ RichObject NoTask ].
    t taskHolding: false.
    t priority > currentTask priority
		ifTrue: [ ^ t ]
		ifFalse: [ ^ currentTask ]! !

!RichRunner methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:11'!
schedule
    currentTask := taskList. 
    [RichObject NoTask == currentTask]
		whileFalse: 
	    [currentTask isTaskHoldingOrWaiting
			ifTrue: [currentTask := currentTask link]
			ifFalse: 
		    [currentTaskIdentity := currentTask identity.
			 tracing ifTrue: [ self trace: currentTaskIdentity ].
		    currentTask := currentTask runTask]]! !

!RichRunner methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:44'!
start
    | workQ |
	 self initTrace.
    self initScheduler.
    self 	createIdler: RichObject Idler
			priority: 0
			work: RichObject NoWork
			state: RichTaskState running.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject Worker
		kind: RichObject WorkPacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject Worker
		kind: RichObject WorkPacketKind.
    self
		createWorker: RichObject Worker
		priority: 1000
		work: workQ
		state: RichTaskState waitingWithPacket.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    self
		createHandler: RichObject HandlerA
		priority: 2000
		work: workQ
		state: RichTaskState waitingWithPacket.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    self
		createHandler: RichObject HandlerB
		priority: 3000
		work: workQ
		state: RichTaskState waitingWithPacket.
    self
		createDevice: RichObject DeviceA
		priority: 4000
		work: RichObject NoWork
		state: RichTaskState waiting.
    self
		createDevice: RichObject DeviceB
		priority: 5000
		work: RichObject NoWork
		state: RichTaskState waiting.
    
	 self schedule.
    ^ (queuePacketCount = 23246) & (holdCount = 9297)
! !

!RichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:40'!
wait 
    currentTask taskWaiting: true.
    ^ currentTask! !

Object subclass: #SomAll
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

Object subclass: #SomBall
	instanceVariableNames: 'x y xVel yVel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomBall methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 13:51'!
bounce
        | xLimit yLimit bounced |
        xLimit  := yLimit := 500.
        bounced := false.
        
        x := x + xVel.
        y := y + yVel.
        (x > xLimit)
            ifTrue: [ x := xLimit. xVel := 0 - xVel abs. bounced := true ].
        (x < 0)
            ifTrue: [ x := 0.      xVel := xVel abs.     bounced := true ].
        (y > yLimit)
            ifTrue: [ y := yLimit. yVel := 0 - yVel abs. bounced := true ].
        (y < 0)
            ifTrue: [ y := 0.      yVel := yVel abs.     bounced := true ].
        ^bounced! !

!SomBall methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 13:58'!
initialize
        x := SomRandom next rem: 500.
        y := SomRandom next rem: 500.
        xVel := (SomRandom next rem: 300) - 150.
        yVel := (SomRandom next rem: 300) - 150.! !

Object subclass: #SomBenchmark
	instanceVariableNames: 'numIterations numWarmUpIterations innerIterations printIndividualRuns maxRuntime total'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomBenchmark methodsFor: 'converting' stamp: 'StefanMarr 4/5/2015 10:47'!
assert: expected equals: value
	expected = value ifFalse: [
		self error: 'Expected value (', expected asString,
						') differs from actual (', value asString, ') benchmark result.'].
	^ true! !

!SomBenchmark methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 10:45'!
benchmark
	self subclassResponsibility! !

!SomBenchmark methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 10:37'!
doRuns
        | i  totalRunTime |
        i := 0.
        totalRunTime := 0.
        
        [ i < numIterations ] whileTrue: [
            | startTime endTime runTime |
            startTime := Time primUTCMicrosecondsClock.
            self innerBenchmarkLoop ifFalse: [
              self error: 'Benchmark failed with incorrect result'. ].
            endTime   := Time primUTCMicrosecondsClock.
            
            runTime := endTime - startTime.
            printIndividualRuns ifTrue: [ self printRun: runTime ].
            
            totalRunTime := totalRunTime + runTime.
            i := i + 1.
            
            maxRuntime ifNotNil: [
                totalRunTime > maxRuntime ifTrue: [
                    numIterations := i.
                    ^ totalRunTime ]].
            
            Smalltalk garbageCollect ].
        
        ^ totalRunTime! !

!SomBenchmark methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 10:32'!
exec
	     ScriptConsole << (('Starting ', self name), ' benchmark ... ').
        self warmup.
        total := self doRuns.! !

!SomBenchmark methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 10:38'!
innerBenchmarkLoop
        | i |
        i := 0.
        [ i < innerIterations ] whileTrue: [
            (self verifyResult: self benchmark) ifFalse: [ ^ false ].
            i := i + 1.
        ].
        ^ true! !

!SomBenchmark methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 10:27'!
innerIterations
	^ innerIterations! !

!SomBenchmark methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 10:28'!
maxRuntime: seconds
        "converted to microseconds i.e. ticks"
        seconds ifNotNil: [
            maxRuntime := seconds * 1000 * 1000].
! !

!SomBenchmark methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 12:53'!
name
	| className |
	"Remove Som prefix"
	className := self class name asString.
	^  className copyFrom: 4 to: className size! !

!SomBenchmark methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 10:27'!
numIterations
	^ numIterations! !

!SomBenchmark methodsFor: 'reporting' stamp: 'StefanMarr 4/5/2015 13:40'!
printRun: runTime
        ScriptConsole << (self name, ': iterations=1 runtime: ', runTime asString, 'us'); cr! !

!SomBenchmark methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 10:29'!
run
        | harness |
        harness := SomAll new.
        harness initialize.
        harness benchmarkClasses: (OrderedCollection with: self class).
        harness printAll: false.
        harness maxRuntime: 3. "seconds"
        harness numIterations: 100.
        harness warmUp: 10.

        harness runBenchmarks.
        harness printTotal.! !

!SomBenchmark methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 10:28'!
run: numIter with: numWarmUpIter and: numInner printAll: aBool
        numIterations       := numIter.
        numWarmUpIterations := numWarmUpIter.
        innerIterations     := numInner.
        printIndividualRuns := aBool.
        
        self exec.! !

!SomBenchmark methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 10:27'!
total
	^ total! !

!SomBenchmark methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 10:45'!
verifyResult: result
	self subclassResponsibility! !

!SomBenchmark methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 10:31'!
warmup
        | numIterationsTmp printIndividualRunsTmp maxRuntimeTmp |
        numWarmUpIterations > 0 ifFalse: [
            ScriptConsole cr.
            ^ self].

        numIterationsTmp := numIterations.
        printIndividualRunsTmp := printIndividualRuns.
        maxRuntimeTmp := maxRuntime.

        numIterations := numWarmUpIterations.
        printIndividualRuns := false.
        maxRuntime := nil.

        ScriptConsole << ' warmup ...'.
        self doRuns.

        numIterations := numIterationsTmp.
        printIndividualRuns := printIndividualRunsTmp.
        maxRuntime := maxRuntimeTmp.

        ScriptConsole << ' completed.'.! !

SomBenchmark subclass: #SomBounce
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomBounce methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:56'!
benchmark
        | ballCount balls bounces |
        SomRandom initialize.
        
        ballCount := 100.
        bounces   := 0.
        balls     := Array new: ballCount.
        1 to: balls size do: [:i | balls at: i put: SomBall new ].
        
        1 to: 50 do: [ :i |
            balls do: [ :ball |
                (ball bounce) ifTrue: [ bounces := bounces + 1 ] ] ].

        ^ bounces! !

!SomBounce methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:34'!
verifyResult: result
      ^ self assert: 1331 equals: result! !

SomBenchmark subclass: #SomDeltaBlue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomDeltaBlue methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 12:55'!
innerBenchmarkLoop
	DBStrength initialize.
	DBPlanner chainTest: innerIterations.
	DBPlanner projectionTest: innerIterations.
	^ true! !

SomBenchmark subclass: #SomFannkuch
	instanceVariableNames: 'timesRotated perm atEnd'
	classVariableNames: 'results'
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomFannkuch class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:25'!
expectedResult: problemSize
      results isNil ifTrue: [
        results := Array new: 12.
        results at:  1 put:  0.
        results at:  2 put:  1.
        results at:  3 put:  2.
        results at:  4 put:  4.
        results at:  5 put:  7.
        results at:  6 put: 10.
        results at:  7 put: 16.
        results at:  8 put: 22.
        results at:  9 put: 30.
        results at: 10 put: 38.
        results at: 11 put: 51.
        results at: 12 put: 65.
      ].
      ^ results at: problemSize! !

!SomFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:23'!
atEnd
        ^atEnd! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:24'!
benchFannkuch
		| result |
      self initialize: self problemSize.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: self problemSize)! !

!SomFannkuch methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 16:31'!
initialize: size
	perm := Array new: size.
	1 to: size do: [:i | perm at: i put: i ].
	timesRotated := Array new: size withAll: 0.
	atEnd := false! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 11:36'!
innerBenchmarkLoop
		| result |
      self initialize: innerIterations.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: innerIterations)! !

!SomFannkuch methodsFor: 'initialize-release' stamp: 'StefanMarr 4/5/2015 22:45'!
makeNext
	| temp remainder |
	"* Generate the next permutation. *"
	2 to: perm size do: [ :r |
		"* Rotate the first r items to the left. *"
		temp := perm at: 1.
		1 to: r - 1 do: [ :i | perm at: i put: (perm at: i + 1) ].
		perm at: r put: temp.

		timesRotated at: r put: (((timesRotated at: r) + 1) rem: r).
		remainder := timesRotated at: r.
		remainder = 0 ifFalse: [ ^self ].

		"* After r rotations, the first r items are in their original positions.
		 Go on rotating the first r+1 items. *"
	].

	"* We are past the final permutation. *"
	atEnd := true.! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:23'!
maxPfannkuchen
        | max permutation |
        max := 0.
        [self atEnd] whileFalse: [
            permutation := self next.
            max := max max: (self pfannkuchen: permutation)].
        ^max! !

!SomFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:24'!
next
        | result |
        result := perm copy.
        self makeNext.
        ^result! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:21'!
pfannkuchen: anArray
	| first complement a b k |
	k := 0.
	[ (first := anArray at: 1) = 1 ] whileFalse: [
		k := k + 1.
		complement := first + 1.

		1 to: first / 2 do: [ :i |
			a := anArray at: i.
			b := anArray at: complement - i.
			anArray at: i put: b.
			anArray at: complement - i put: a.
	] ].
	^ k! !

SomBenchmark subclass: #SomFieldLoop
	instanceVariableNames: 'counter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-LanguageFeatures'!

!SomFieldLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:04'!
benchmark
        | iter |
        counter := 0.
        iter := 20000.

        [ iter > 0 ] whileTrue: [
          iter := iter - 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
          counter := counter + 1.
        ].
        
        ^ counter! !

!SomFieldLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:04'!
verifyResult: result
      ^ 600000 = result! !

SomBenchmark subclass: #SomGraphSearch
	instanceVariableNames: 'graphNodes graphMask updatingGraphMask graphVisited cost graphEdges k firstCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-GraphSearch'!

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedNoOfNodes
	^ 3000000! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedTotalCost
	^ 26321966! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxInitEdges
	^ 4! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxNodes 
	^ 1 << 31! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
MaxWeight
	^ 1! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinEdges
	^ 2
! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:35'!
MinNodes
	^ 20! !

!SomGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinWeight
	^ 1! !

!SomGraphSearch methodsFor: 'helper' stamp: 'StefanMarr 4/3/2015 15:34'!
breadthFirstSearch: noOfNodes
    | stop |
    stop := true.
    [stop] whileTrue: [
      stop := false.
      
      1 to: noOfNodes do: [:tid |
        (graphMask at: tid) ifTrue: [
          graphMask at: tid put: false.
          (graphNodes at: tid) starting 
              to: ((graphNodes at: tid) noOfEdges + (graphNodes at: tid) starting) - 1
              do: [:i |
            | id |
            id := graphEdges at: i.
            (graphVisited at: id) ifFalse: [
              cost at: id put: (cost at: tid) + 1.
              updatingGraphMask at: id put: true.
            ]
          ]
        ]
      ].
      
      1 to: noOfNodes do: [:tid |
        (updatingGraphMask at: tid) ifTrue: [
          graphMask    at: tid put: true.
          graphVisited at: tid put: true.
          stop := true.
          updatingGraphMask at: tid put: false
        ]
      ]
    ]! !

!SomGraphSearch methodsFor: 'initialization' stamp: 'StefanMarr 4/5/2015 20:27'!
initializeGraph: noOfNodes
    | source graph totalEdges |
    graphNodes        := Array new: noOfNodes.
    graphMask         := Array new: noOfNodes withAll: false.
    updatingGraphMask := Array new: noOfNodes withAll: false.
    graphVisited      := Array new: noOfNodes withAll: false.
    cost              := Array new: noOfNodes withAll: -1.

    source := 1.

    graph := Array new: noOfNodes.
    1 to: noOfNodes do: [:i | graph at: i put: OrderedCollection new ].

    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (SomJenkinsRandom random
                       rem: SomGraphSearch MaxInitEdges - SomGraphSearch MinEdges + 1) abs
                   + SomGraphSearch MinEdges.
      1 to: noOfEdges do: [:j |
        | nodeId weight |
        nodeId := (SomJenkinsRandom random rem: noOfNodes) abs + 1.
        weight := (SomJenkinsRandom random rem: (
                    SomGraphSearch MaxWeight - SomGraphSearch MinWeight + 1)) abs
                  + SomGraphSearch MinWeight.
        (graph at: i)      add: (GSEdge newWith: nodeId and: weight).
        (graph at: nodeId) add: (GSEdge newWith:      i and: weight).
      ]
    ].

    totalEdges := 0.
    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (graph at: i) size.
      graphNodes at: i put: (GSNode newWith: totalEdges + 1 and: noOfEdges).
      totalEdges := totalEdges + noOfEdges
    ].

    graphMask    at: source put: true.
    graphVisited at: source put: true.

    graphEdges := Array new: totalEdges withAll: 0.
    
    k := 1.
    graph do: [:i |
      i do: [:j |
        graphEdges at: k put: j dest.
        k := k + 1.
      ]
    ].
    cost at: source put: 0.! !

!SomGraphSearch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 20:29'!
innerBenchmarkLoop
    | noOfNodes |
    SomJenkinsRandom seed: 49734321.
    noOfNodes := SomGraphSearch ExpectedNoOfNodes / 1000 * innerIterations.

    self initializeGraph: noOfNodes.
    self breadthFirstSearch: noOfNodes.
    ^ self verifyResult: cost! !

!SomGraphSearch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 20:57'!
verifyResult: result
    | totalCost |
    cost size = (SomGraphSearch ExpectedNoOfNodes // 1000 * innerIterations)
      ifFalse: [ ^ false ].

    totalCost := 0.    
    cost do: [:c | totalCost := totalCost + c].

    cost size = SomGraphSearch ExpectedNoOfNodes
      ifTrue:  [
        totalCost = SomGraphSearch ExpectedTotalCost ifFalse: [
          self error: 'ERROR: the total cost obtained for ', cost length asString,
              ' nodes is ', totalCost asString, ' while the expected cost is ',
              SomGraphSearch ExpectedTotalCost 
        ]]
      ifFalse: [
        firstCost == nil
          ifTrue: [
            firstCost := totalCost.
            ^ true ]
          ifFalse: [
            ^ firstCost = totalCost
          ]
      ].

    ^ true! !

SomBenchmark subclass: #SomIntegerLoop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-LanguageFeatures'!

!SomIntegerLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:05'!
benchmark
        | bounds a |
        bounds := 20000.
        bounds negated to: bounds by: 1 do: [:value | a := value-value].
        ^ a! !

!SomIntegerLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:05'!
verifyResult: result
      ^ 0 = result! !

SomBenchmark subclass: #SomJson
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Json'!

!SomJson class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/2/2015 22:45'!
RapBenchmarkMinified
	^ '{"head":{"requestCounter":4},"operations":[["destroy","w54"],["set","w2",{"activeControl":"w99"}],["set","w21",{"customVariant":"variant_navigation"}],["set","w28",{"customVariant":"variant_selected"}],["set","w53",{"children":["w95"]}],["create","w95","rwt.widgets.Composite",{"parent":"w53","style":["NONE"],"bounds":[0,0,1008,586],"children":["w96","w97"],"tabIndex":-1,"clientArea":[0,0,1008,586]}],["create","w96","rwt.widgets.Label",{"parent":"w95","style":["NONE"],"bounds":[10,30,112,26],"tabIndex":-1,"customVariant":"variant_pageHeadline","text":"TableViewer"}],["create","w97","rwt.widgets.Composite",{"parent":"w95","style":["NONE"],"bounds":[0,61,1008,525],"children":["w98","w99","w226","w228"],"tabIndex":-1,"clientArea":[0,0,1008,525]}],["create","w98","rwt.widgets.Text",{"parent":"w97","style":["LEFT","SINGLE","BORDER"],"bounds":[10,10,988,32],"tabIndex":22,"activeKeys":["#13","#27","#40"]}],["listen","w98",{"KeyDown":true,"Modify":true}],["create","w99","rwt.widgets.Grid",{"parent":"w97","style":["SINGLE","BORDER"],"appearance":"table","indentionWidth":0,"treeColumn":-1,"markupEnabled":false}],["create","w100","rwt.widgets.ScrollBar",{"parent":"w99","style":["HORIZONTAL"]}],["create","w101","rwt.widgets.ScrollBar",{"parent":"w99","style":["VERTICAL"]}],["set","w99",{"bounds":[10,52,988,402],"children":[],"tabIndex":23,"activeKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"],"cancelKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"]}],["listen","w99",{"MouseDown":true,"MouseUp":true,"MouseDoubleClick":true,"KeyDown":true}],["set","w99",{"itemCount":118,"itemHeight":28,"itemMetrics":[[0,0,50,3,0,3,44],[1,50,50,53,0,53,44],[2,100,140,103,0,103,134],[3,240,180,243,0,243,174],[4,420,50,423,0,423,44],[5,470,50,473,0,473,44]],"columnCount":6,"headerHeight":35,"headerVisible":true,"linesVisible":true,"focusItem":"w108","selection":["w108"]}],["listen","w99",{"Selection":true,"DefaultSelection":true}],["set","w99",{"enableCellToolTip":true}],["listen","w100",{"Selection":true}],["set","w101",{"visibility":true}],["listen","w101",{"Selection":true}],["create","w102","rwt.widgets.GridColumn",{"parent":"w99","text":"Nr.","width":50,"moveable":true}],["listen","w102",{"Selection":true}],["create","w103","rwt.widgets.GridColumn",{"parent":"w99","text":"Sym.","index":1,"left":50,"width":50,"moveable":true}],["listen","w103",{"Selection":true}],["create","w104","rwt.widgets.GridColumn",{"parent":"w99","text":"Name","index":2,"left":100,"width":140,"moveable":true}],["listen","w104",{"Selection":true}],["create","w105","rwt.widgets.GridColumn",{"parent":"w99","text":"Series","index":3,"left":240,"width":180,"moveable":true}],["listen","w105",{"Selection":true}],["create","w106","rwt.widgets.GridColumn",{"parent":"w99","text":"Group","index":4,"left":420,"width":50,"moveable":true}],["listen","w106",{"Selection":true}],["create","w107","rwt.widgets.GridColumn",{"parent":"w99","text":"Period","index":5,"left":470,"width":50,"moveable":true}],["listen","w107",{"Selection":true}],["create","w108","rwt.widgets.GridItem",{"parent":"w99","index":0,"texts":["1","H","Hydrogen","Nonmetal","1","1"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w109","rwt.widgets.GridItem",{"parent":"w99","index":1,"texts":["2","He","Helium","Noble gas","18","1"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w110","rwt.widgets.GridItem",{"parent":"w99","index":2,"texts":["3","Li","Lithium","Alkali metal","1","2"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w111","rwt.widgets.GridItem",{"parent":"w99","index":3,"texts":["4","Be","Beryllium","Alkaline earth metal","2","2"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w112","rwt.widgets.GridItem",{"parent":"w99","index":4,"texts":["5","B","Boron","Metalloid","13","2"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w113","rwt.widgets.GridItem",{"parent":"w99","index":5,"texts":["6","C","Carbon","Nonmetal","14","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w114","rwt.widgets.GridItem",{"parent":"w99","index":6,"texts":["7","N","Nitrogen","Nonmetal","15","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w115","rwt.widgets.GridItem",{"parent":"w99","index":7,"texts":["8","O","Oxygen","Nonmetal","16","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w116","rwt.widgets.GridItem",{"parent":"w99","index":8,"texts":["9","F","Fluorine","Halogen","17","2"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w117","rwt.widgets.GridItem",{"parent":"w99","index":9,"texts":["10","Ne","Neon","Noble gas","18","2"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w118","rwt.widgets.GridItem",{"parent":"w99","index":10,"texts":["11","Na","Sodium","Alkali metal","1","3"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w119","rwt.widgets.GridItem",{"parent":"w99","index":11,"texts":["12","Mg","Magnesium","Alkaline earth metal","2","3"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w120","rwt.widgets.GridItem",{"parent":"w99","index":12,"texts":["13","Al","Aluminium","Poor metal","13","3"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w121","rwt.widgets.GridItem",{"parent":"w99","index":13,"texts":["14","Si","Silicon","Metalloid","14","3"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w122","rwt.widgets.GridItem",{"parent":"w99","index":14,"texts":["15","P","Phosphorus","Nonmetal","15","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w123","rwt.widgets.GridItem",{"parent":"w99","index":15,"texts":["16","S","Sulfur","Nonmetal","16","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w124","rwt.widgets.GridItem",{"parent":"w99","index":16,"texts":["17","Cl","Chlorine","Halogen","17","3"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w125","rwt.widgets.GridItem",{"parent":"w99","index":17,"texts":["18","Ar","Argon","Noble gas","18","3"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w126","rwt.widgets.GridItem",{"parent":"w99","index":18,"texts":["19","K","Potassium","Alkali metal","1","4"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w127","rwt.widgets.GridItem",{"parent":"w99","index":19,"texts":["20","Ca","Calcium","Alkaline earth metal","2","4"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w128","rwt.widgets.GridItem",{"parent":"w99","index":20,"texts":["21","Sc","Scandium","Transition metal","3","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w129","rwt.widgets.GridItem",{"parent":"w99","index":21,"texts":["22","Ti","Titanium","Transition metal","4","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w130","rwt.widgets.GridItem",{"parent":"w99","index":22,"texts":["23","V","Vanadium","Transition metal","5","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w131","rwt.widgets.GridItem",{"parent":"w99","index":23,"texts":["24","Cr","Chromium","Transition metal","6","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w132","rwt.widgets.GridItem",{"parent":"w99","index":24,"texts":["25","Mn","Manganese","Transition metal","7","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w133","rwt.widgets.GridItem",{"parent":"w99","index":25,"texts":["26","Fe","Iron","Transition metal","8","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w134","rwt.widgets.GridItem",{"parent":"w99","index":26,"texts":["27","Co","Cobalt","Transition metal","9","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w135","rwt.widgets.GridItem",{"parent":"w99","index":27,"texts":["28","Ni","Nickel","Transition metal","10","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w136","rwt.widgets.GridItem",{"parent":"w99","index":28,"texts":["29","Cu","Copper","Transition metal","11","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w137","rwt.widgets.GridItem",{"parent":"w99","index":29,"texts":["30","Zn","Zinc","Transition metal","12","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w138","rwt.widgets.GridItem",{"parent":"w99","index":30,"texts":["31","Ga","Gallium","Poor metal","13","4"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w139","rwt.widgets.GridItem",{"parent":"w99","index":31,"texts":["32","Ge","Germanium","Metalloid","14","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w140","rwt.widgets.GridItem",{"parent":"w99","index":32,"texts":["33","As","Arsenic","Metalloid","15","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w141","rwt.widgets.GridItem",{"parent":"w99","index":33,"texts":["34","Se","Selenium","Nonmetal","16","4"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w142","rwt.widgets.GridItem",{"parent":"w99","index":34,"texts":["35","Br","Bromine","Halogen","17","4"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w143","rwt.widgets.GridItem",{"parent":"w99","index":35,"texts":["36","Kr","Krypton","Noble gas","18","4"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w144","rwt.widgets.GridItem",{"parent":"w99","index":36,"texts":["37","Rb","Rubidium","Alkali metal","1","5"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w145","rwt.widgets.GridItem",{"parent":"w99","index":37,"texts":["38","Sr","Strontium","Alkaline earth metal","2","5"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w146","rwt.widgets.GridItem",{"parent":"w99","index":38,"texts":["39","Y","Yttrium","Transition metal","3","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w147","rwt.widgets.GridItem",{"parent":"w99","index":39,"texts":["40","Zr","Zirconium","Transition metal","4","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w148","rwt.widgets.GridItem",{"parent":"w99","index":40,"texts":["41","Nb","Niobium","Transition metal","5","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w149","rwt.widgets.GridItem",{"parent":"w99","index":41,"texts":["42","Mo","Molybdenum","Transition metal","6","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w150","rwt.widgets.GridItem",{"parent":"w99","index":42,"texts":["43","Tc","Technetium","Transition metal","7","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w151","rwt.widgets.GridItem",{"parent":"w99","index":43,"texts":["44","Ru","Ruthenium","Transition metal","8","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w152","rwt.widgets.GridItem",{"parent":"w99","index":44,"texts":["45","Rh","Rhodium","Transition metal","9","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w153","rwt.widgets.GridItem",{"parent":"w99","index":45,"texts":["46","Pd","Palladium","Transition metal","10","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w154","rwt.widgets.GridItem",{"parent":"w99","index":46,"texts":["47","Ag","Silver","Transition metal","11","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w155","rwt.widgets.GridItem",{"parent":"w99","index":47,"texts":["48","Cd","Cadmium","Transition metal","12","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w156","rwt.widgets.GridItem",{"parent":"w99","index":48,"texts":["49","In","Indium","Poor metal","13","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w157","rwt.widgets.GridItem",{"parent":"w99","index":49,"texts":["50","Sn","Tin","Poor metal","14","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w158","rwt.widgets.GridItem",{"parent":"w99","index":50,"texts":["51","Sb","Antimony","Metalloid","15","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w159","rwt.widgets.GridItem",{"parent":"w99","index":51,"texts":["52","Te","Tellurium","Metalloid","16","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w160","rwt.widgets.GridItem",{"parent":"w99","index":52,"texts":["53","I","Iodine","Halogen","17","5"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w161","rwt.widgets.GridItem",{"parent":"w99","index":53,"texts":["54","Xe","Xenon","Noble gas","18","5"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w162","rwt.widgets.GridItem",{"parent":"w99","index":54,"texts":["55","Cs","Caesium","Alkali metal","1","6"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w163","rwt.widgets.GridItem",{"parent":"w99","index":55,"texts":["56","Ba","Barium","Alkaline earth metal","2","6"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w164","rwt.widgets.GridItem",{"parent":"w99","index":56,"texts":["57","La","Lanthanum","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w165","rwt.widgets.GridItem",{"parent":"w99","index":57,"texts":["58","Ce","Cerium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w166","rwt.widgets.GridItem",{"parent":"w99","index":58,"texts":["59","Pr","Praseodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w167","rwt.widgets.GridItem",{"parent":"w99","index":59,"texts":["60","Nd","Neodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w168","rwt.widgets.GridItem",{"parent":"w99","index":60,"texts":["61","Pm","Promethium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w169","rwt.widgets.GridItem",{"parent":"w99","index":61,"texts":["62","Sm","Samarium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w170","rwt.widgets.GridItem",{"parent":"w99","index":62,"texts":["63","Eu","Europium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w171","rwt.widgets.GridItem",{"parent":"w99","index":63,"texts":["64","Gd","Gadolinium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w172","rwt.widgets.GridItem",{"parent":"w99","index":64,"texts":["65","Tb","Terbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w173","rwt.widgets.GridItem",{"parent":"w99","index":65,"texts":["66","Dy","Dysprosium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w174","rwt.widgets.GridItem",{"parent":"w99","index":66,"texts":["67","Ho","Holmium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w175","rwt.widgets.GridItem",{"parent":"w99","index":67,"texts":["68","Er","Erbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w176","rwt.widgets.GridItem",{"parent":"w99","index":68,"texts":["69","Tm","Thulium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w177","rwt.widgets.GridItem",{"parent":"w99","index":69,"texts":["70","Yb","Ytterbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w178","rwt.widgets.GridItem",{"parent":"w99","index":70,"texts":["71","Lu","Lutetium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w179","rwt.widgets.GridItem",{"parent":"w99","index":71,"texts":["72","Hf","Hafnium","Transition metal","4","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w180","rwt.widgets.GridItem",{"parent":"w99","index":72,"texts":["73","Ta","Tantalum","Transition metal","5","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w181","rwt.widgets.GridItem",{"parent":"w99","index":73,"texts":["74","W","Tungsten","Transition metal","6","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w182","rwt.widgets.GridItem",{"parent":"w99","index":74,"texts":["75","Re","Rhenium","Transition metal","7","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w183","rwt.widgets.GridItem",{"parent":"w99","index":75,"texts":["76","Os","Osmium","Transition metal","8","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w184","rwt.widgets.GridItem",{"parent":"w99","index":76,"texts":["77","Ir","Iridium","Transition metal","9","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w185","rwt.widgets.GridItem",{"parent":"w99","index":77,"texts":["78","Pt","Platinum","Transition metal","10","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w186","rwt.widgets.GridItem",{"parent":"w99","index":78,"texts":["79","Au","Gold","Transition metal","11","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w187","rwt.widgets.GridItem",{"parent":"w99","index":79,"texts":["80","Hg","Mercury","Transition metal","12","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w188","rwt.widgets.GridItem",{"parent":"w99","index":80,"texts":["81","Tl","Thallium","Poor metal","13","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w189","rwt.widgets.GridItem",{"parent":"w99","index":81,"texts":["82","Pb","Lead","Poor metal","14","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w190","rwt.widgets.GridItem",{"parent":"w99","index":82,"texts":["83","Bi","Bismuth","Poor metal","15","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w191","rwt.widgets.GridItem",{"parent":"w99","index":83,"texts":["84","Po","Polonium","Metalloid","16","6"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w192","rwt.widgets.GridItem",{"parent":"w99","index":84,"texts":["85","At","Astatine","Halogen","17","6"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w193","rwt.widgets.GridItem",{"parent":"w99","index":85,"texts":["86","Rn","Radon","Noble gas","18","6"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w194","rwt.widgets.GridItem",{"parent":"w99","index":86,"texts":["87","Fr","Francium","Alkali metal","1","7"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w195","rwt.widgets.GridItem",{"parent":"w99","index":87,"texts":["88","Ra","Radium","Alkaline earth metal","2","7"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w196","rwt.widgets.GridItem",{"parent":"w99","index":88,"texts":["89","Ac","Actinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w197","rwt.widgets.GridItem",{"parent":"w99","index":89,"texts":["90","Th","Thorium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w198","rwt.widgets.GridItem",{"parent":"w99","index":90,"texts":["91","Pa","Protactinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w199","rwt.widgets.GridItem",{"parent":"w99","index":91,"texts":["92","U","Uranium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w200","rwt.widgets.GridItem",{"parent":"w99","index":92,"texts":["93","Np","Neptunium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w201","rwt.widgets.GridItem",{"parent":"w99","index":93,"texts":["94","Pu","Plutonium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w202","rwt.widgets.GridItem",{"parent":"w99","index":94,"texts":["95","Am","Americium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w203","rwt.widgets.GridItem",{"parent":"w99","index":95,"texts":["96","Cm","Curium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w204","rwt.widgets.GridItem",{"parent":"w99","index":96,"texts":["97","Bk","Berkelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w205","rwt.widgets.GridItem",{"parent":"w99","index":97,"texts":["98","Cf","Californium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w206","rwt.widgets.GridItem",{"parent":"w99","index":98,"texts":["99","Es","Einsteinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w207","rwt.widgets.GridItem",{"parent":"w99","index":99,"texts":["100","Fm","Fermium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w208","rwt.widgets.GridItem",{"parent":"w99","index":100,"texts":["101","Md","Mendelevium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w209","rwt.widgets.GridItem",{"parent":"w99","index":101,"texts":["102","No","Nobelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w210","rwt.widgets.GridItem",{"parent":"w99","index":102,"texts":["103","Lr","Lawrencium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w211","rwt.widgets.GridItem",{"parent":"w99","index":103,"texts":["104","Rf","Rutherfordium","Transition metal","4","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w212","rwt.widgets.GridItem",{"parent":"w99","index":104,"texts":["105","Db","Dubnium","Transition metal","5","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w213","rwt.widgets.GridItem",{"parent":"w99","index":105,"texts":["106","Sg","Seaborgium","Transition metal","6","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w214","rwt.widgets.GridItem",{"parent":"w99","index":106,"texts":["107","Bh","Bohrium","Transition metal","7","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w215","rwt.widgets.GridItem",{"parent":"w99","index":107,"texts":["108","Hs","Hassium","Transition metal","8","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w216","rwt.widgets.GridItem",{"parent":"w99","index":108,"texts":["109","Mt","Meitnerium","Transition metal","9","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w217","rwt.widgets.GridItem",{"parent":"w99","index":109,"texts":["110","Ds","Darmstadtium","Transition metal","10","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w218","rwt.widgets.GridItem",{"parent":"w99","index":110,"texts":["111","Rg","Roentgenium","Transition metal","11","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w219","rwt.widgets.GridItem",{"parent":"w99","index":111,"texts":["112","Uub","Ununbium","Transition metal","12","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w220","rwt.widgets.GridItem",{"parent":"w99","index":112,"texts":["113","Uut","Ununtrium","Poor metal","13","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w221","rwt.widgets.GridItem",{"parent":"w99","index":113,"texts":["114","Uuq","Ununquadium","Poor metal","14","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w222","rwt.widgets.GridItem",{"parent":"w99","index":114,"texts":["115","Uup","Ununpentium","Poor metal","15","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w223","rwt.widgets.GridItem",{"parent":"w99","index":115,"texts":["116","Uuh","Ununhexium","Poor metal","16","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w224","rwt.widgets.GridItem",{"parent":"w99","index":116,"texts":["117","Uus","Ununseptium","Halogen","17","7"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w225","rwt.widgets.GridItem",{"parent":"w99","index":117,"texts":["118","Uuo","Ununoctium","Noble gas","18","7"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w226","rwt.widgets.Composite",{"parent":"w97","style":["BORDER"],"bounds":[10,464,988,25],"children":["w227"],"tabIndex":-1,"clientArea":[0,0,986,23]}],["create","w227","rwt.widgets.Label",{"parent":"w226","style":["NONE"],"bounds":[10,10,966,3],"tabIndex":-1,"text":"Hydrogen (H)"}],["create","w228","rwt.widgets.Label",{"parent":"w97","style":["WRAP"],"bounds":[10,499,988,16],"tabIndex":-1,"foreground":[150,150,150,255],"font":[["Verdana","Lucida Sans","Arial","Helvetica","sans-serif"],10,false,false],"text":"Shortcuts: [CTRL+F] - Filter | Sort by: [CTRL+R] - Number, [CTRL+Y] - Symbol, [CTRL+N] - Name, [CTRL+S] - Series, [CTRL+G] - Group, [CTRL+E] - Period"}],["set","w1",{"focusControl":"w99"}],["call","rwt.client.BrowserNavigation","addToHistory",{"entries":[["tableviewer","TableViewer"]]}]]}'
    ! !

!SomJson methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 18:17'!
benchmark
	^ (JSJsonParser with: SomJson RapBenchmarkMinified) parse! !

!SomJson methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 12:59'!
exec
	JSJsonLiteral initialize.
	super exec.! !

!SomJson methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:00'!
verifyResult: result
	result class = JSParseException ifTrue: [ result println ].
	result isObject ifFalse: [^ false].
	(result asObject at: 'head') isObject ifFalse: [^ false].
	(result asObject at: 'operations') isArray ifFalse: [^ false].
	^ (result asObject at: 'operations') asArray size = 156! !

SomBenchmark subclass: #SomMandelbrot
	instanceVariableNames: 'firstResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomMandelbrot methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:28'!
innerBenchmarkLoop
	^ self verifyResult: (self mandelbrot: innerIterations)! !

!SomMandelbrot methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:46'!
mandelbrot: size
        | sum byteAcc bitNum y |
        sum     := 0.
        byteAcc := 0.
        bitNum  := 0.
  
        y := 0.
        
        [y < size] whileTrue: [
            | ci x |
            ci := (2.0 * y / size) - 1.0.
            x  := 0.
            
            [x < size] whileTrue: [
                | zr zrzr zi zizi cr escape z notDone |
                zrzr := zr := 0.0.
                zizi := zi := 0.0.
                cr   := (2.0 * x / size) - 1.5.
                
                z := 0.
                notDone := true.
                escape := 0.
                [notDone and: [z < 50]] whileTrue: [
                    zr := zrzr - zizi + cr.
                    zi := 2.0 * zr * zi + ci.
            
                    "preserve recalculation"
                    zrzr := zr * zr.
                    zizi := zi * zi.
            
                    (zrzr + zizi > 4.0) ifTrue: [
                        notDone := false.
                        escape  := 1.
                    ].
                    z := z + 1.
                ].
  
                byteAcc := (byteAcc << 1) + escape.
                bitNum  := bitNum + 1.
  
                " Code is very similar for these cases, but using separate blocks
                  ensures we skip the shifting when it's unnecessary,
                  which is most cases. "
                bitNum = 8 
                    ifTrue: [
                      sum := sum bitXor: byteAcc.
                      byteAcc := 0.
                      bitNum  := 0. ]
                    ifFalse: [
                      (x = (size - 1)) ifTrue: [
                          byteAcc := byteAcc << (8 - bitNum).
                          sum := sum bitXor: byteAcc.
                          byteAcc := 0.
                          bitNum  := 0. ]].
                x := x + 1.
            ].
            y := y + 1.
        ].
        
        ^ sum! !

!SomMandelbrot methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:01'!
verifyResult: result
	   innerIterations = 750 ifTrue: [
        ^ result = 50
      ].
      firstResult isNil ifTrue: [
        firstResult := result.
      ].
      ^ result = firstResult! !

SomBenchmark subclass: #SomNBody
	instanceVariableNames: 'expectedEnergy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-NBody'!

!SomNBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:07'!
initialize
	NBBody initialize! !

!SomNBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 14:10'!
innerBenchmarkLoop
	     | bodies |
        bodies := NBNBodySystem new.
        
        1 to: innerIterations do: [:i |
            bodies advance: 0.01.
        ].
        
        innerIterations = 250000 ifTrue: [
          ^ bodies energy = -0.1690859889909308
        ].
        expectedEnergy isNil ifTrue: [
          expectedEnergy := bodies energy.
          ^ true
        ].
        
        ^ expectedEnergy = bodies energy.! !

SomBenchmark subclass: #SomPageRank
	instanceVariableNames: 'firstResult'
	classVariableNames: 'expected'
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:14'!
DFactor
	^ 0.85! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:40'!
ExpectedPageRanks
    expected ifNotNil: [ ^ expected ].
    
    expected := OrderedCollection new: 5000.
    self privExRank01: expected;
         privExRank02: expected;
         privExRank03: expected;
         privExRank04: expected;
         privExRank05: expected;
         privExRank06: expected;
         privExRank07: expected;
         privExRank08: expected;
         privExRank09: expected;
         privExRank10: expected;
         privExRank11: expected;
         privExRank12: expected;
         privExRank13: expected;
         privExRank14: expected;
         privExRank15: expected;
         privExRank16: expected;
         privExRank17: expected;
         privExRank18: expected;
         privExRank19: expected;
         privExRank20: expected;
         privExRank21: expected;
         privExRank22: expected;
         privExRank23: expected;
         privExRank24: expected.
    
    expected := expected asArray.
    ^ expected! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:20'!
privExRank01: exp
	exp add:
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0001937753607433594; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00026861193950589846; add:  0.0011626495265046287; add:  0.00033125968546875005; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.002419297210359863; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00028382917108593755; add:  0.00005550000000000001; add: 
       0.00022373208234375002; add:  0.0009500131460436332; add:  0.0004673045821320703; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00029348190124218755; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0002522648462121094; add:  0.0002007225; add:  0.0005721072315653125; add: 
       0.0006549038427949217; add:  0.00027208502253146485; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00022515500625; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00020761099425802734; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0014816283407223048; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00018305958234375; add:  0.00014299384687500004; add:  0.00011125893750000003; add: 
       0.00015412125000000001; add:  0.00005550000000000001; add:  0.0005866682663756834; add: 
       0.0009038160999648827; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00014299384687500004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00012968268750000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000214412971875; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00016827375000000002; add:  0.00005550000000000001; add:  0.0007612028507183201; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0017911603607849024; add:  0.00012435000000000001; add:  0.0002074907896875; add: 
       0.000030000000000000004; add:  0.000620176162217578; add:  0.0016205421327844336; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0001772368043671875; add: 
       0.0002207814375; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0009383260308160549; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00011727375000000003; add:  0.00025645517495238287; add: 
       0.00046964726999218764; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00048092971362552716; add:  0.00022518875783285154; add:  0.0002320883721796875; add: 
       0.00014550189499218754; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.0001801089375; add:  0.0003218540343750001; add: 
       0.000030000000000000004; add:  0.00024264642949335937; add:  0.00014659875000000002; add: 
       0.00009885000000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00012435000000000001; add:  0.00005550000000000001; add:  0.00009885000000000002; add: 
       0.00009559875000000003; add:  0.00007717500000000002; add:  0.00020376888060937495; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00011727375000000003; add:  0.00005550000000000001; add: 
       0.00023148805436718755; add:  0.00005550000000000001; add:  0.0003663848267566407; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00020837250000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000128175; add:  0.00005550000000000001; add: 
       0.0007321965646351562; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.0002918212500000001; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00018287250000000003; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005117003058394531; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005327917869365431; add:  0.000030000000000000004; add: 
       0.00013675893750000003; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0008628989988383205; add:  0.00005550000000000001; add:  0.000030000000000000004. 

! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:21'!
privExRank02: exp
	exp add:
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000126919125; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00011125893750000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.00027766269375; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00018287250000000003; add:  0.000030000000000000004; add:  0.00006825; add: 
       0.000030000000000000004; add:  0.00012052500000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.0005574534602777341; add:  0.00011402250000000001; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000540919912277207; add:  0.00008100000000000002; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00013588458234375003; add:  0.000030000000000000004; add:  0.00020486333264062499; add: 
       0.0010616644951779884; add:  0.00005550000000000001; add:  0.00012109875000000002; add: 
       0.00013894875000000002; add:  0.0006629172614616794; add:  0.0015453146662926173; add: 
       0.000030000000000000004; add:  0.0002057215678125; add:  0.00014277375000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00012457009687500003; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00022662231875; add:  0.0002800117218750001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012457009687500003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0009413663007857225; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00025216269375; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00014299384687500004; add:  0.000030000000000000004; add:  0.0004204378892502149; add: 
       0.00022315833234375003; add:  0.001512352057544629; add:  0.00040963230479023444; add: 
       0.00026866422187500004; add:  0.00009885000000000002; add:  0.000030000000000000004; add: 
       0.0004487230533789453; add:  0.000030000000000000004; add:  0.0001568163804609375; add: 
       0.00014985; add:  0.00007717500000000002; add:  0.00037312038758925784; add: 
       0.00010267500000000002; add:  0.00007717500000000002; add:  0.000935572065964551; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006538908974685352; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0006925555784296876; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000146025; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00004275000000000001; add:  0.0002412005625; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000351187703481504; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.00019815573560605466; add:  0.0001356975; add: 
       0.00007717500000000002; add:  0.00013588458234375003; add:  0.0019199758003062113; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006042323629701756; add: 
       0.000030000000000000004; add:  0.0008090710683361913; add:  0.000030000000000000004; add: 
       0.00021544875; add:  0.000030000000000000004; add:  0.0003507081512421876; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00028123619308710943; add:  0.00007717500000000002; add:  0.00022293583249218752; add: 
       0.000030000000000000004; add:  0.00028254900584279304; add:  0.0004448098933681447; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.000406985079798047; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00020730370734375002; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.0006841020911458787; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00019508699999999998; add: 
       0.00023373894375; add:  0.000269050764124043; add:  0.00007717500000000002; add: 
       0.0012665335189376758; add:  0.0003229170826183594; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00026877685218750007; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00035339283501253916; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000220427784375; add:  0.000030000000000000004; add:  0.000132; add:  0.00025887973491310547; add: 
       0.0010559972458375393; add:  0.00007717500000000002; add:  0.000030000000000000004.! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:22'!
privExRank03: exp
	exp add:
      0.00005550000000000001; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.0007952039924323437; add:  0.00009559875000000003; add:  0.0007619092408695311; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00005550000000000001; add: 
       0.0004741191615037304; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00044242997182148444; add:  0.00014277375000000002; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0008103090240004296; add:  0.000030000000000000004; add:  0.0008127957153562891; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.0005670410421640625; add: 
       0.00012435000000000001; add:  0.0009996132445867678; add:  0.00005550000000000001; add: 
       0.000190168846875; add:  0.00019637074171875002; add:  0.00005550000000000001; add: 
       0.0006413808275615429; add:  0.0008067955708537303; add:  0.0007001598184784763; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00019364019374999998; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00024570768750000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000816114721648008; add: 
       0.00016996851984375; add:  0.000251748159375; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00045582617944294916; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0004087417901358985; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.0007096632942357614; add: 
       0.0004450802223830273; add:  0.000030000000000000004; add:  0.00013788125625000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0007781633581341993; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0007831720377038671; add: 
       0.00007717500000000002; add:  0.000146025; add:  0.000030000000000000004; add: 
       0.0004440335706074609; add:  0.0002797916250000001; add:  0.00014534287500000002; add: 
       0.000161417596875; add:  0.0009436753876707029; add:  0.00026090069906250005; add: 
       0.000661540765294531; add:  0.00029711647746210945; add:  0.0002448333323437501; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00021968458249218753; add:  0.00007717500000000002; add:  0.00018669750000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0004852117315546876; add: 
       0.0003194149033965235; add:  0.000030000000000000004; add:  0.000770995975586406; add: 
       0.000582465439638496; add:  0.001229452233367832; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000166433693109375; add:  0.000969294173404297; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0001513576875; add:  0.00009559875000000003; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00027891726984375005; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0003005925113290822; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005980535795580271; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0006150758829812889; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00015007009687500002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00063777321360625; add: 
       0.000030000000000000004; add:  0.00041378858300664085; add:  0.0002291377273359375; add: 
       0.00008100000000000002; add:  0.0007787174847144531; add:  0.00011125893750000003; add: 
       0.000319367505031465; add:  0.00037540608941236337; add:  0.000030000000000000004; add: 
       0.0008391853330165822; add:  0.000030000000000000004; add:  0.0001952814375; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.0004302220856250001; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00022148922187500001; add:  0.00008100000000000002; add: 
       0.000168493846875; add:  0.00005550000000000001; add:  0.00016444875000000002; add: 
       0.00022096851984374999; add:  0.0004387228943070313; add:  0.000030000000000000004; add: 
       0.00018994875000000002; add:  0.00043396908058710945; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00119860742323042; add:  0.00013675893750000003; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00008100000000000002; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.0002644775398359375; add: 
       0.0007738206560650585; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0007043146423551758; add:  0.00005550000000000001; add:  0.000753936463079121; add: 
       0.00014277375000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0007755696951406249; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:29'!
privExRank04: exp
	exp add:
             0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0007849164249744726; add:  0.00019180253983593749; add: 
       0.00013675893750000003; add:  0.000030000000000000004; add:  0.00017895411089179688; add: 
       0.00013293393750000003; add:  0.000030000000000000004; add:  0.0009846043482525976; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0003370130432812501; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006119601669933592; add: 
       0.0005546408271638474; add:  0.00035661147136054695; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00018305958234375; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0004162326721509962; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000140230284375; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00012435000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0001752401304609375; add:  0.00012435000000000001; add: 
       0.00005550000000000001; add:  0.0009916779323594143; add:  0.000030000000000000004; add: 
       0.0001356975; add:  0.0005381491251261719; add:  0.000594984439271953; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.0009741378132010936; add: 
       0.0010289986178938673; add:  0.00009885000000000002; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0003377064978837697; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00014985; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00047936216204097644; add:  0.00012109875000000002; add: 
       0.000944214872861504; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0003911262409296876; add:  0.00016444875000000002; add: 
       0.0004658189423039455; add:  0.0006191960454895896; add:  0.00012109875000000002; add: 
       0.00008100000000000002; add:  0.00018994875; add:  0.0003503022762421876; add: 
       0.00007717500000000002; add:  0.00014659875000000002; add:  0.000030000000000000004; add: 
       0.000294136229749043; add:  0.0001546089375; add:  0.000030000000000000004; add: 
       0.00048512037514843756; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0009038993515082423; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00016792009687500002; add:  0.00047329793096896486; add: 
       0.001073084842359102; add:  0.000227504034375; add:  0.00007717500000000002; add: 
       0.0004113979149621094; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000994461009736816; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00009559875000000003; add:  0.00022421976984375; add:  0.0004580116392566408; add: 
       0.00042668597373246103; add:  0.000030000000000000004; add:  0.0005213088860603709; add: 
       0.00029239500000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00011727375000000003; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.0004510455397947461; add: 
       0.00014277375000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.002259585483123663; add:  0.0010017437556216994; add:  0.00032997952022560553; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0007017341088929103; add: 
       0.00005550000000000001; add:  0.0005571497378849411; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00022170931875; add: 
       0.0006954863535522461; add:  0.00005550000000000001; add:  0.0012527846540595113; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0005184859828281249; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00013293393750000003; add:  0.00007717500000000002; add:  0.0008999596475350778; add: 
       0.00031492419618537123; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0012884279283853029; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0007648574488383199; add:  0.0008473654450535153.! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:29'!
privExRank05: exp
	exp add:
           0.00008100000000000002; add:  0.000030000000000000004; add:  0.000180329034375; add: 
       0.000170842875; add:  0.00005550000000000001; add:  0.000401158314638496; add: 
       0.0006134211817580272; add:  0.00015881305436718754; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.0005238088503046875; add:  0.00005550000000000001; add:  0.00012109875000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00012109875000000002; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.0001605102271875; add:  0.0005087270201025195; add: 
       0.0009967971475125393; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.0006039423526103709; add:  0.0009677339486394532; add:  0.000030000000000000004; add: 
       0.0007675520473719141; add:  0.0007538680901701758; add:  0.00008100000000000002; add: 
       0.0006169301652431443; add:  0.00009885000000000002; add:  0.0005449327357591993; add: 
       0.00011125893750000003; add:  0.00012435000000000001; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00011125893750000003; add:  0.00021132074186718748; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.000254291625; add:  0.00005550000000000001; add: 
       0.000140230284375; add:  0.00005550000000000001; add:  0.0005397573664441406; add: 
       0.000030000000000000004; add:  0.0002286781021875; add:  0.00011727375000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00012109875000000002; add: 
       0.00014985; add:  0.0003482766015246095; add:  0.00007717500000000002; add: 
       0.00015843393750000003; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.00042410386925802743; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00037117269868064466; add:  0.00005550000000000001; add: 
       0.0001806826875; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00025942602609375004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.000245927784375; add: 
       0.00005550000000000001; add:  0.0001768576875; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00012435000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00038169142341406263; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00016978143750000002; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.000218017875; add: 
       0.0003311542154954884; add:  0.0023047579029954004; add:  0.0005710822497378906; add: 
       0.00007717500000000002; add:  0.0003164610867890626; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00039183545686406263; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.0006218280803472657; add:  0.00027158680436718753; add: 
       0.0005329226223534762; add:  0.000030000000000000004; add:  0.0001546089375; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.0013023196241036914; add:  0.000030000000000000004; add:  0.0005814199422881052; add: 
       0.000030000000000000004; add:  0.0010607720917741798; add:  0.00011125893750000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0003044889569777344; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.0008152654201896286; add: 
       0.00005550000000000001; add:  0.000140230284375; add:  0.0007082080488527344; add: 
       0.0007379601076354103; add:  0.000408482320607461; add:  0.00012968268750000001; add: 
       0.0010253114960741213; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000146025; add:  0.00020560893750000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00016119750000000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00031838485247324223; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0014520803604533988; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:30'!
privExRank06: exp
	exp add:
             0.00005550000000000001; add:  0.00009885000000000002; add:  0.000364157824359375; add: 
       0.00005550000000000001; add:  0.0013332881297266016; add:  0.0005557404117852929; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0009105278934790038; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0007129847634453126; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00013675893750000003; add: 
       0.000950365240418213; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00014919574171875; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0001768576875; add: 
       0.00043941458861267575; add:  0.00005550000000000001; add:  0.00028618575000000004; add: 
       0.00020968190109375; add:  0.00012968268750000001; add:  0.00012109875000000002; add: 
       0.0005675717694396287; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002826102781975489; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000345610471875; add:  0.000030000000000000004; add: 
       0.0006355078163115429; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.0003739622193515626; add:  0.00014299384687500004; add:  0.00022441180436718748; add: 
       0.000030000000000000004; add:  0.00044359985994126954; add:  0.0002641154176562501; add: 
       0.000030000000000000004; add:  0.000206951353125; add:  0.00005550000000000001; add: 
       0.00004275000000000001; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00035032903437500005; add:  0.00008100000000000002; add: 
       0.0008822237984673634; add:  0.0007134373373503516; add:  0.000030000000000000004; add: 
       0.0008928574627982618; add:  0.00018287250000000003; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00016827375000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00015865403437500001; add:  0.00008100000000000002; add:  0.00013293393750000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000511693358096836; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0008382409215566016; add: 
       0.000030000000000000004; add:  0.00011402250000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00014277375000000002; add:  0.000590603701889238; add: 
       0.00011402250000000001; add:  0.000030000000000000004; add:  0.0003913641587121094; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00012435000000000001; add: 
       0.000030000000000000004; add:  0.0005719241044744334; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00018612375; add:  0.00041990431397560547; add:  0.000128175; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00056602218509332; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00014919574171875; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0007010716836625781; add:  0.000030000000000000004; add: 
       0.00027271537500000005; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00015851355328125002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0017785457481467384; add:  0.00035147874703125013; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.00017087074171875; add:  0.00005550000000000001; add:  0.00029875788750000006; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00018287250000000003; add: 
       0.00020730370734374997; add:  0.000030000000000000004; add:  0.00046078811633689466; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.0011980866815906645; add:  0.0001897543125; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00024398593546875; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0005523386141103321; add: 
       0.000030000000000000004; add:  0.0011369696555575588; add:  0.00044302649175994153; add: 
       0.00016225893750000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00009885000000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00034647523371210935; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00008100000000000002.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:32'!
privExRank07: exp
	exp add:
             0.000030000000000000004; add:  0.0006266011364933593; add:  0.0007460569827269042; add: 
       0.000152419125; add:  0.0005574828435624999; add:  0.00045754291748888653; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00012435000000000001; add: 
       0.0006514320518563474; add:  0.0007781550062722654; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.000030000000000000004; add:  0.00016978143750000002; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.00021112870734374997; add: 
       0.000030000000000000004; add:  0.0007241425462692384; add:  0.000030000000000000004; add: 
       0.00043975632939673826; add:  0.00026377172812500006; add:  0.0003192690972632813; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.001060316398582637; add: 
       0.000030000000000000004; add:  0.00016827375; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.0002944702747961329; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000348277539984375; add:  0.0001546089375; add:  0.0016954224484599999; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000180329034375; add: 
       0.00005550000000000001; add:  0.00024112459341996093; add:  0.000030000000000000004; add: 
       0.00041604693152460934; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005496041375411911; add:  0.00019251787500000001; add: 
       0.0005949961275392772; add:  0.0001551826875; add:  0.000254291625; add: 
       0.00011727375000000003; add:  0.00007717500000000002; add:  0.00012109875000000002; add: 
       0.00005550000000000001; add:  0.00015354144375; add:  0.00015007009687500002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00027656824171875004; add: 
       0.00022955245734375; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00032534906796093756; add:  0.00012457009687500003; add:  0.000215668846875; add: 
       0.00013894875000000002; add:  0.000205829034375; add:  0.0002736906531408399; add: 
       0.00010267500000000002; add:  0.00005550000000000001; add:  0.0007959996360539453; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00012052500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000224252784375; add:  0.00005550000000000001; add:  0.00017087074171875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0003375011800228516; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00014624509687500003; add:  0.00005550000000000001; add: 
       0.00017152500000000004; add:  0.000030000000000000004; add:  0.0003609669341552931; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0002042114773359375; add: 
       0.0008721219106832425; add:  0.0006369682754531251; add:  0.0001356975; add: 
       0.00005550000000000001; add:  0.00024838187461171876; add:  0.000713347958461465; add: 
       0.00007717500000000002; add:  0.00017303268749999998; add:  0.00008100000000000002; add: 
       0.0005128700965591992; add:  0.00019470905663185548; add:  0.00009885000000000002; add: 
       0.00008100000000000002; add:  0.00026435153437500005; add:  0.000030000000000000004; add: 
       0.00028910833249218753; add:  0.00005550000000000001; add:  0.00024367850573554694; add: 
       0.00015843393750000003; add:  0.000030000000000000004; add:  0.0003614906134310743; add: 
       0.000030000000000000004; add:  0.00035817465068654306; add:  0.00019702661074335936; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00031682620734375006; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.0009339972378849416; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00037407066244818355; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00028621361671875; add:  0.000030000000000000004; add:  0.00017557009687500002; add: 
       0.000030000000000000004; add:  0.00017598333234375; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005464359416179294; add: 
       0.00012968268750000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009885000000000002; add:  0.000454723037081924; add:  0.000030000000000000004; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00026667045761835943; add:  0.0010615728509209819; add:  0.00043192417839472676; add: 
       0.00014277375000000002; add:  0.0016763097454050194; add:  0.0004163827734375001; add: 
       0.00017174509687500002; add:  0.00007717500000000002; add:  0.0005071352788982225; add: 
       0.000030000000000000004; add:  0.0004724088400752538; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.0013976675910372072; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.00016827375; add:  0.0009015184824459963.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:32'!
privExRank08: exp
	exp add:
             0.00011402250000000001; add:  0.000030000000000000004; add:  0.00020620815124218746; add: 
       0.0006429872559689648; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00027610687500000006; add: 
       0.000030000000000000004; add:  0.00030289569021933597; add:  0.0006080530205542576; add: 
       0.00011727375000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0010484356591858599; add:  0.000030000000000000004; add:  0.0004212905427699219; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00043143789891640633; add: 
       0.00014659875000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002032035485179687; add:  0.0007551751218261719; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0003794698669933595; add: 
       0.00014659875000000002; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00011727375000000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0004934760543513475; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000128175; add: 
       0.00005550000000000001; add:  0.0008547118390340233; add:  0.000030000000000000004; add: 
       0.0014346082973754881; add:  0.00021695643750000001; add:  0.00005550000000000001; add: 
       0.0015211104302111527; add:  0.00011727375000000003; add:  0.00005550000000000001; add: 
       0.0004392343555509962; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.00005550000000000001; add:  0.0009416894236694923; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00022873191724904294; add: 
       0.0008242737845741408; add:  0.00025836458859375004; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.00031687600301942386; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.00015843393750000003; add:  0.000390462103125; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0004362053242109375; add:  0.00019035592921875; add: 
       0.000030000000000000004; add:  0.0005769638100933201; add:  0.00006825; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00022241513046093752; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.001006261613162637; add: 
       0.0007857088789003512; add:  0.000030000000000000004; add:  0.00025498721718750006; add: 
       0.00005550000000000001; add:  0.0011432981682779491; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.0006757280227431446; add: 
       0.000600739772793574; add:  0.00014919574171875; add:  0.000030000000000000004; add: 
       0.0004217083662265626; add:  0.000030000000000000004; add:  0.0021483529248317776; add: 
       0.00005550000000000001; add:  0.0006842449449634961; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00013588458234375003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.0003539867951484376; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.0005256094171827733; add: 
       0.000030000000000000004; add:  0.00014985; add:  0.00007717500000000002; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0006064650511017775; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00023480038124999998; add:  0.000030000000000000004; add: 
       0.000386993224139668; add:  0.0007025938102384176; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0008015986250978318; add: 
       0.0011856587198157474; add:  0.001220542597781563; add:  0.00005550000000000001; add: 
       0.001103845676174756; add:  0.000030000000000000004; add:  0.0005001217209945313; add: 
       0.000164668846875; add:  0.000030000000000000004; add:  0.00021766422187500002; add: 
       0.00009559875000000003; add:  0.0002708821616396681; add:  0.0008902189805322072; add: 
       0.0002728723524621094; add:  0.0001928969918671875; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00008100000000000002; add:  0.00012435000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00020673125625000002; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.00012109875000000002; add:  0.000030000000000000004; add:  0.0001795858324921875; add: 
       0.00014659875000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006738107530478319; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.0005546722821167576; add: 
       0.00013675893750000003; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0007719215799593356; add:  0.00009559875000000003; add:  0.00021162375000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00012968268750000001; add: 
       0.00005550000000000001; add:  0.00027164788125; add:  0.00005550000000000001; add: 
       0.0014103401638943945; add:  0.00015154476984375004; add:  0.0005737931960558594; add: 
       0.0008436621533431052; add:  0.00012457009687500003; add:  0.00019145643750000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:33'!
privExRank09: exp
	exp add:
             0.00010267500000000002; add:  0.000030000000000000004; add:  0.0002807485055871094; add: 
       0.0005178079962573242; add:  0.000030000000000000004; add:  0.00042718429824335937; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00009885000000000002; add: 
       0.00008100000000000002; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.0009962234120309377; add:  0.000030000000000000004; add:  0.0002712076875; add: 
       0.0007484966368463475; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00021782324186718748; add:  0.0007975849074420704; add:  0.00005550000000000001; add: 
       0.0014440971395321093; add:  0.0004298739782575976; add:  0.000030000000000000004; add: 
       0.00016827375; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0010549562700050393; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000126919125; add:  0.000183092596875; add:  0.00045689434595333993; add: 
       0.000030000000000000004; add:  0.0006202840925404099; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00041851013033476574; add:  0.0009673196422284764; add:  0.00011402250000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00024139500000000002; add:  0.00015881305436718754; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0005228328479950585; add:  0.000030000000000000004; add: 
       0.00021283083234375; add:  0.000030000000000000004; add:  0.0007436318898950389; add: 
       0.0004801294072429297; add:  0.00010267500000000002; add:  0.000502228642298047; add: 
       0.000030000000000000004; add:  0.000572949911904863; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000269925119280293; add:  0.0007819586751550391; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0007033901825952341; add: 
       0.00007717500000000002; add:  0.000165730284375; add:  0.00008100000000000002; add: 
       0.0010263752493400197; add:  0.000030000000000000004; add:  0.00014299384687500004; add: 
       0.000030000000000000004; add:  0.0002240326875; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0001584339375; add:  0.00045708025931367194; add: 
       0.00007717500000000002; add:  0.00047436583249218747; add:  0.000030000000000000004; add: 
       0.0002730541168671876; add:  0.000030000000000000004; add:  0.00016225893750000003; add: 
       0.000030000000000000004; add:  0.00040197340380373055; add:  0.000030000000000000004; add: 
       0.000261367875; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00013894875000000002; add:  0.000030000000000000004; add: 
       0.0002513864773359376; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00047612431834921886; add:  0.00007717500000000002; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.0006952168221796875; add:  0.00012109875000000002; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.001107573809079551; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00018431305436718753; add:  0.000030000000000000004; add:  0.000165730284375; add: 
       0.00030354806250000013; add:  0.00040802521613302745; add:  0.0007408627669822461; add: 
       0.0003570662890816407; add:  0.00028091394375000006; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.0001356975; add:  0.0004396857337185352; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00011125893750000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00038478386950781257; add:  0.000030000000000000004; add:  0.0007459325985840232; add: 
       0.0003567720534166603; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.0001356975; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00017798000625; add:  0.0014359547638245706; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00046789477015955094; add:  0.000030000000000000004; add:  0.00010650000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000226919852484375; add: 
       0.00022297125; add:  0.000596758121936328; add:  0.000030000000000000004; add: 
       0.0006366992817095507; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.00007717500000000002; add:  0.00017196519375; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.0002011018125; add: 
       0.000030000000000000004; add:  0.0002167318951183594; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.0008290133266141014; add:  0.000030000000000000004; add:  0.000183580284375; add: 
       0.00023110893750000005; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00028930645988302735; add:  0.0001649910962121094; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0003763601051910157; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.0001573725; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:33'!
privExRank10: exp
	exp add:
       0.0004523801286034764; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0004951815598534765; add:  0.00008100000000000002; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.001175440344576426; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00035443770990195323; add:  0.000030000000000000004; add: 
       0.0003747032963148926; add:  0.00005550000000000001; add:  0.0005116082614710938; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00021331851984375; add: 
       0.00007717500000000002; add:  0.0008077201393309957; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000994534956437754; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.000030000000000000004; add:  0.0009117984778980079; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00036226694577816426; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00032663461605585945; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.0006578930123802537; add:  0.00013293393750000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000146025; add:  0.0006895069584232615; add: 
       0.00012968268750000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00034956950625; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00047335328654826193; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0003485537326737892; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00020124834327152344; add: 
       0.000030000000000000004; add:  0.0005462430679701755; add:  0.0010715045568435355; add: 
       0.00007717500000000002; add:  0.0001356975; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00005550000000000001; add:  0.001050337302045796; add: 
       0.0005798148140722265; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.0005860187924074219; add: 
       0.000695229028248086; add:  0.00036895237500000005; add:  0.00005550000000000001; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00022698333234375003; add:  0.000030000000000000004; add:  0.00018994875000000002; add: 
       0.00030363530370429697; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0012613613445382034; add:  0.0009061000272912988; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00044870342936718753; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00030216397748437506; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0003228767387781642; add:  0.00009885000000000002; add:  0.00016338125625; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005762014488482618; add:  0.00005550000000000001; add: 
       0.000252516346875; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00043890087460225595; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00016701787500000002; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.0009819741968800392; add:  0.00047788861671875; add: 
       0.00005550000000000001; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.001424334953569922; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003072850223830275; add:  0.00019267689499218754; add: 
       0.0002809656928125; add:  0.000174094125; add:  0.0001650225; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.0011707606836439263; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00014277375000000002; add:  0.0011823230263219532; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00013244625000000002; add:  0.0006514792329670507; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0003684399653628907; add: 
       0.000030000000000000004; add:  0.00038739278765625003; add:  0.000030000000000000004; add: 
       0.00022297125000000003; add:  0.00028660922160039077; add:  0.00022708709554921871; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0006567602159249608; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.0005183511222142771; add: 
       0.00201874207261627; add:  0.00037233539805243184; add:  0.00005550000000000001; add: 
       0.0004911306645736522; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0005789639421744336; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00023881851984375; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000258336721875; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00046646993601609374; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:34'!
privExRank11: exp
	exp add:
             0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00015865403437500001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00017174509687500002; add: 
       0.00005550000000000001; add:  0.00019164351984375002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0004902321882890625; add:  0.00011402250000000001; add: 
       0.00012968268750000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00021589500000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0010836219592849413; add:  0.0010891681019760942; add:  0.000030000000000000004; add: 
       0.00012435000000000001; add:  0.00005550000000000001; add:  0.0007079486145866796; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.0003419444918671875; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.001505804961977207; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00032973896826562506; add:  0.00013675893750000003; add: 
       0.00007717500000000002; add:  0.0007428987645248242; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.0004649452599635351; add: 
       0.000128175; add:  0.000030000000000000004; add:  0.0012289260041186134; add: 
       0.0005547659253248241; add:  0.0012735475711920707; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0006304702346261715; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.0006962683298203127; add: 
       0.0015407160072108007; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000575346237916621; add:  0.0008075947471042188; add:  0.00005550000000000001; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.0001513576875; add: 
       0.00007717500000000002; add:  0.00016444875; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.0012133833776340236; add:  0.00016701787500000002; add:  0.0006743516353132419; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00028358187446328134; add:  0.0005657072281472655; add:  0.000128175; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00010267500000000002; add: 
       0.0005714230341351562; add:  0.0003404881216429688; add:  0.0005203748888714452; add: 
       0.000030000000000000004; add:  0.00039713555163551777; add:  0.0003546995317031251; add: 
       0.0004783518928479491; add:  0.000030000000000000004; add:  0.00012052500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00044235231887617196; add:  0.000030000000000000004; add: 
       0.00016444875; add:  0.00019609428398671873; add:  0.000126919125; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0001677; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00034297342150898445; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00016792009687500002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.000128175; add:  0.00040291446016097664; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00037065834674240237; add:  0.000030000000000000004; add: 
       0.0006004148802589451; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.00019016884687500003; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00008100000000000002; add:  0.0006253264456486521; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0010081750473936526; add: 
       0.0001584339375; add:  0.00015007009687500002; add:  0.000425316405686543; add: 
       0.00032775693480585947; add:  0.00016792009687500002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000770613605453125; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0010328207671150588; add:  0.0001801089375; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0005880984259152342; add: 
       0.00009885000000000002; add:  0.00005550000000000001; add:  0.0002693696017992188; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005425831398122849; add: 
       0.000030000000000000004; add:  0.00028088092921875; add:  0.00010267500000000002; add: 
       0.0006764355701484372; add:  0.00008100000000000002; add:  0.0003437430909375001; add: 
       0.0009308938220033006; add:  0.0009178192984247847; add:  0.0007719799011481251; add: 
       0.00007717500000000002; add:  0.00013675893750000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00009885000000000002; add: 
       0.00014810643750000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006161561722175781; add: 
       0.002341625584141152; add:  0.0002463233892566407; add:  0.0014496255159246877; add: 
       0.00039820903892660153; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.0004494569376072462; add:  0.00005550000000000001.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:34'!
privExRank12: exp
	exp add:
             0.0007375958831105469; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00019691513046093753; add:  0.000030000000000000004; add:  0.0006960112701089452; add: 
       0.000030000000000000004; add:  0.0001356975; add:  0.00010267500000000002; add: 
       0.00012457009687500003; add:  0.00009559875000000003; add:  0.00028669488604804693; add: 
       0.0005773723220977929; add:  0.0025455574471148447; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0008289915908865819; add: 
       0.00012435000000000001; add:  0.000254291625; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003034902985243946; add:  0.00031878986671875003; add: 
       0.0006200913580648829; add:  0.00089866348489084; add:  0.000030000000000000004; add: 
       0.00024847125000000003; add:  0.000030000000000000004; add:  0.00044653863884943365; add: 
       0.0009588353839884179; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0004802001794933595; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0005993897213728708; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00012435000000000001; add:  0.0006565863834296874; add:  0.0004414859974607228; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.00011125893750000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00046621688976466807; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0003956887727812501; add:  0.000187405284375; add:  0.00008100000000000002; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000322784383514668; add:  0.00005550000000000001; add: 
       0.00013588458234375003; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00013293393750000003; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002027230162402734; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00011727375000000003; add: 
       0.000030000000000000004; add:  0.0002919278240625; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0011767052391037309; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00015361125; add: 
       0.0017282723083981836; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0007795432040711617; add:  0.00007717500000000002; add:  0.00008100000000000002; add: 
       0.00014985; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0003018932018828127; add:  0.00035541894550021494; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0007141020085688866; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00011125893750000003; add: 
       0.001132409535121133; add:  0.00024089480953125002; add:  0.00022514327545312499; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0004311059424464845; add: 
       0.000128175; add:  0.00019035592921875; add:  0.000030000000000000004; add:  0.0001768576875; add: 
       0.0009238328865187112; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00008100000000000002; add:  0.00012435000000000001; add:  0.00012109875000000002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0001965018949921875; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.000340432358860547; add: 
       0.000030000000000000004; add:  0.00009885000000000002; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.0006406874962545115; add: 
       0.00047452478709895517; add:  0.0010634564139578517; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000164668846875; add: 
       0.00007717500000000002; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00014076403125000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00031425467936718754; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00013244625000000002; add:  0.00011125893750000003; add: 
       0.00019459416468749997; add:  0.001184899122552129; add:  0.0006817205711949704; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.0005271594965196095; add:  0.0010400870586408399; add:  0.0004752608268892381; add: 
       0.000030000000000000004; add:  0.00020129625; add:  0.000030000000000000004; add: 
       0.00064671966671541; add:  0.0002982153750000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003997243869443556; add:  0.00048734821359374986; add: 
       0.0004759093776406251; add:  0.00011727375000000003; add:  0.000030000000000000004; add: 
       0.0005440875227589844; add:  0.00005550000000000001; add:  0.00019582394999999997; add: 
       0.00037308581183710947; add:  0.00005550000000000001; add:  0.00005550000000000001.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:35'!
privExRank13: exp
	exp add:
            0.00008100000000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0004962196445816407; add:  0.00011727375000000003; add:  0.000164668846875; add: 
       0.00005550000000000001; add:  0.00013588458234375003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00020380225558710936; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.00022728393750000005; add: 
       0.00011402250000000001; add:  0.000030000000000000004; add:  0.0006183836193906247; add: 
       0.0001730326875; add:  0.00008100000000000002; add:  0.0012089610287725784; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0010155726653473343; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00016119750000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000192737971875; add: 
       0.000030000000000000004; add:  0.0006722068841278708; add:  0.0010718436839313866; add: 
       0.000030000000000000004; add:  0.001128313168464766; add:  0.002193723830047618; add: 
       0.00022640958234375002; add:  0.000030000000000000004; add:  0.00025480590886054686; add: 
       0.0008077473013638865; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.0005865207417187499; add:  0.000030000000000000004; add:  0.00017579625; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00022315833234375003; add:  0.00014277375000000002; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.0003722072014756055; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00017598333234375; add:  0.0006452517980099412; add: 
       0.000174094125; add:  0.00024649398358593755; add:  0.000030000000000000004; add: 
       0.00011125893750000003; add:  0.00044401822041662117; add:  0.000030000000000000004; add: 
       0.00012109875000000002; add:  0.00008100000000000002; add:  0.0004345341423233984; add: 
       0.0016198246023303518; add:  0.00007717500000000002; add:  0.00024390100558710937; add: 
       0.00023174226984375002; add:  0.00016827375000000002; add:  0.000030000000000000004; add: 
       0.00023969287499999998; add:  0.000030000000000000004; add:  0.00029862255421875006; add: 
       0.0002884303875000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0012822416701354098; add:  0.00011727375000000003; add: 
       0.00019875278437500002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00013293393750000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000231320375; add:  0.00011727375000000003; add: 
       0.000030000000000000004; add:  0.00013293393750000003; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.0006375577699699217; add: 
       0.00013293393750000003; add:  0.0005119848858394532; add:  0.00012968268750000001; add: 
       0.00005550000000000001; add:  0.00009885000000000002; add:  0.00019747261913185545; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0002687487898359375; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00012052500000000002; add: 
       0.00018287250000000003; add:  0.000030000000000000004; add:  0.00027437842921875004; add: 
       0.00025469880421875006; add:  0.00021870000000000003; add:  0.001054292033974219; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.0002522608324921875; add: 
       0.00017904144375; add:  0.0000880125; add:  0.00008100000000000002; add: 
       0.0008665907698721879; add:  0.00009559875000000003; add:  0.00011727375000000003; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000195405039984375; add: 
       0.000030000000000000004; add:  0.00015154476984375004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.0004193817696888867; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00020620815124218752; add:  0.00011402250000000001; add: 
       0.00034031658501562504; add:  0.00008100000000000002; add:  0.00016225893750000003; add: 
       0.000030000000000000004; add:  0.00016444875000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00012457009687500003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0013239555484927151; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0003249621588382814; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.002044314609656622; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0002445528608917969; add: 
       0.000030000000000000004; add:  0.0005265423747665818; add:  0.000140230284375; add: 
       0.00005550000000000001; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.00035159226984375006; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0006776933182608983; add:  0.000030000000000000004; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:35'!
privExRank14: exp
	exp add:
       0.000030000000000000004; add:  0.00011402250000000001; add:  0.000030000000000000004; add: 
       0.00011125893750000003; add:  0.00022071992883046875; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00015412125000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.0003944472271875; add:  0.00014277375000000002; add: 
       0.000030000000000000004; add:  0.0003893728637121095; add:  0.00005550000000000001; add: 
       0.0005825809400868945; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00019546851984375; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00022476706823554686; add:  0.00009559875000000003; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.0004692964517755665; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.0006707026050272654; add: 
       0.000030000000000000004; add:  0.000128175; add:  0.0010464823826843753; add: 
       0.00013293393750000003; add:  0.00005550000000000001; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00009885000000000002; add:  0.00009559875000000003; add:  0.00011125893750000003; add: 
       0.00005550000000000001; add:  0.0011688491673253515; add:  0.00038341672349904315; add: 
       0.00033088877587617195; add:  0.000030000000000000004; add:  0.00040050305411792956; add: 
       0.0003604310280146681; add:  0.00016827375000000002; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00060983427609375; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00014985; add:  0.000030000000000000004; add:  0.00048470548947291976; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00016827375; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00020129625; add: 
       0.00008100000000000002; add:  0.00018820518749999997; add:  0.00022042778437499996; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.0007755538548972265; add: 
       0.000030000000000000004; add:  0.0006086423521652343; add:  0.00010267500000000002; add: 
       0.0006501657417822068; add:  0.0002758531021875; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0009579667596042187; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00021845292093749998; add:  0.00040177039710841806; add:  0.00043681785095626963; add: 
       0.0003784534574986133; add:  0.000030000000000000004; add:  0.00026525375625; add: 
       0.000030000000000000004; add:  0.0002016704146875; add:  0.0016676698321384962; add: 
       0.0016026092253459765; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.00005550000000000001; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000268820838890625; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00040044269215550797; add: 
       0.00013788125625000002; add:  0.0010699961603220117; add:  0.000030000000000000004; add: 
       0.000146025; add:  0.00008100000000000002; add:  0.0001801089375; add:  0.000927249592168359; add: 
       0.00025420596890625007; add:  0.00030216799120429697; add:  0.0003564819935984766; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005913996994322457; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00031262326397560547; add: 
       0.0008478974921164453; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00024811154062500003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00008100000000000002; add:  0.00046281358580802727; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0004620340538085939; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.0004924743943451367; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006910466454180467; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0008784156453262499; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0003290759292187501; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00009885000000000002; add:  0.0002467391346093751; add:  0.000770629669361074; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00035637197413966804; add: 
       0.0006510181400392773; add:  0.0001932; add:  0.000030000000000000004; add: 
       0.00013293393750000003; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005881109660193553; add:  0.0006697311012689257; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0019048124731323435; add: 
       0.000030000000000000004; add:  0.0012570390195532622; add:  0.000817076816654082; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00037593731782833994; add:  0.0003176617982433595; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.0009558905795487891; add:  0.0001356975; add: 
       0.00007717500000000002; add:  0.002363161256569453; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.0003379288264756056; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:35'!
privExRank15: exp
	exp add:
             0.00010267500000000002; add:  0.000030000000000000004; add:  0.0002601929146875; add: 
       0.0002584977130570313; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0006118566301355858; add:  0.00013675893750000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00019853268750000003; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00011125893750000003; add: 
       0.00007717500000000002; add:  0.00017174509687500002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00015843393750000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00012968268750000001; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00031165253983593755; add:  0.0004989153507442577; add: 
       0.0002587445625; add:  0.00005550000000000001; add:  0.00038396702839960947; add: 
       0.00045063995922013693; add:  0.0009222551727412307; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00012457009687500003; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.00014330527412167969; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00047010862846427726; add:  0.00035199944906250005; add: 
       0.0003710352612187501; add:  0.00011727375000000003; add:  0.0012424059491165434; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.0004125731167599415; add: 
       0.00008100000000000002; add:  0.0002748134751562501; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00023476058810156252; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000332562522531465; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00015354144375; add: 
       0.00012109875000000002; add:  0.0002386314375; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.0009038323984436134; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.0009495415982433593; add:  0.00039800205690646495; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00039053699136197274; add: 
       0.00005550000000000001; add:  0.00012968268750000001; add:  0.000030000000000000004; add: 
       0.0006424087503536033; add:  0.00011727375000000003; add:  0.0007974356282954882; add: 
       0.00005550000000000001; add:  0.00013293393750000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.002784270476832207; add:  0.00007717500000000002; add: 
       0.0010764892763057425; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002833907783996095; add:  0.0007036298547836328; add: 
       0.00005550000000000001; add:  0.00046026382205075194; add:  0.0013872956926727733; add: 
       0.00007717500000000002; add:  0.00009559875000000003; add:  0.00008100000000000002; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.0003395341507054688; add:  0.000030000000000000004; add:  0.0005002082528993947; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.0007778580435433202; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0007483684404247458; add: 
       0.00011727375000000003; add:  0.00009559875000000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.0006657233743404492; add:  0.000030000000000000004; add:  0.000548018287406992; add: 
       0.00042246069565507797; add:  0.001355053843909375; add:  0.0008567852761221873; add: 
       0.00016119750000000003; add:  0.00016701787500000002; add:  0.00007717500000000002; add: 
       0.00007717500000000002; add:  0.00020443397718749996; add:  0.00011402250000000001; add: 
       0.00009559875000000003; add:  0.00014277375000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00017152500000000004; add:  0.000030000000000000004; add:  0.0005834868354039061; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00028394420437500005; add: 
       0.00010267500000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003260878243371094; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000128175; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000192737971875; add: 
       0.00009885000000000002; add:  0.0005593131134316015; add:  0.00007717500000000002.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:36'!
privExRank16: exp
	exp add:
             0.0003434709373674024; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00033521453983593764; add:  0.00010267500000000002; add:  0.00011727375000000003; add: 
       0.0010148410295049998; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00019744738492187498; add:  0.00019875278437500002; add:  0.00013031062500000001; add: 
       0.000030000000000000004; add:  0.00020403750000000002; add:  0.0005788974922679687; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00017521644374999997; add: 
       0.0003113064375; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00040593012538185554; add:  0.00015119241562500002; add: 
       0.0020183828489022167; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0010626563778712694; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.0010527559032889554; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00011727375000000003; add:  0.00014985; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0008434553828328515; add: 
       0.0004977945566349413; add:  0.00005550000000000001; add:  0.0006722223484265624; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0006747054431943555; add: 
       0.000210941625; add:  0.0008963466727096389; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0001605102271875; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00023775102609375; add:  0.0001932; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0013960562834876764; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00010267500000000002; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00009885000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00043246640566427745; add:  0.0002664178528851563; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005632257407151952; add:  0.00047896735528576165; add: 
       0.000632508809124043; add:  0.000030000000000000004; add:  0.0016506434779794729; add: 
       0.00024059509687500002; add:  0.000030000000000000004; add:  0.000192517875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00019853268750000003; add: 
       0.00005550000000000001; add:  0.0012807454091226561; add:  0.0006797589345724413; add: 
       0.00019668014138871093; add:  0.00038820125014843763; add:  0.00009885000000000002; add: 
       0.00005550000000000001; add:  0.00018010893750000003; add:  0.0012861574391040038; add: 
       0.00019382727609374998; add:  0.000803065772465078; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0004803804035068555; add:  0.000030000000000000004; add: 
       0.0004718426704453124; add:  0.00005550000000000001; add:  0.00010267500000000002; add: 
       0.0004208408546006056; add:  0.0001828725; add:  0.001204916585841094; add: 
       0.000687985769325869; add:  0.00021094162500000002; add:  0.00016827375; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00011727375000000003; add: 
       0.0003829687037402735; add:  0.000030000000000000004; add:  0.0002555475; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00011402250000000001; add:  0.00027699795761835936; add:  0.00010267500000000002; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012457009687500003; add:  0.000030000000000000004; add: 
       0.000181390471875; add:  0.00007717500000000002; add:  0.00026633888046093747; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0002201722699921875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.00030823986686718754; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00011125893750000003; add: 
       0.00005550000000000001; add:  0.00021162375000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00010650000000000001; add:  0.00019303215886054686; add: 
       0.00011125893750000003; add:  0.000030000000000000004; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00026469084375000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00019486807662404297; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.001238094285992715; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.0009582389058128515; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00024953268749999997; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00011125893750000003; add:  0.0001513576875; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010267500000000002.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:36'!
privExRank17: exp
	exp add:
            0.00014257931250000002; add:  0.000030000000000000004; add:  0.001362380613464414; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00023734384687500003; add: 
       0.00031125545949474615; add:  0.00016887406781249998; add:  0.000030000000000000004; add: 
       0.00017962125; add:  0.000030000000000000004; add:  0.00032175329238164074; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.001066234109780723; add:  0.0001395225; add: 
       0.00010267500000000002; add:  0.00013293393750000003; add:  0.0004844613860603709; add: 
       0.0002699217076183594; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005309502408943944; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0001356975; add: 
       0.00011402250000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0010622049395273834; add:  0.00023004750000000002; add: 
       0.000030000000000000004; add:  0.0002491371986964844; add:  0.000030000000000000004; add: 
       0.00012435000000000001; add:  0.00005550000000000001; add:  0.00022138175531249998; add: 
       0.00007717500000000002; add:  0.0017936928663086918; add:  0.00042966960194765636; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0009841252989410156; add: 
       0.000261941499592793; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0001932; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.00043106509355255866; add:  0.0010437505899590234; add: 
       0.0002792762970273438; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00018047532249609375; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00048641498056345683; add:  0.00024958621359375; add: 
       0.0003643005830224805; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0007504075031519922; add:  0.000030000000000000004; add:  0.0002680099635937501; add: 
       0.00005550000000000001; add:  0.0003932948035558594; add:  0.000030000000000000004; add: 
       0.000172545; add:  0.0003614907562500001; add:  0.000030000000000000004; add: 
       0.00022164824186718748; add:  0.0006228411419443552; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005299478300855074; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00012968268750000001; add:  0.0003085340538527344; add:  0.00019853268750000003; add: 
       0.00005550000000000001; add:  0.0009653476063435351; add:  0.000030000000000000004; add: 
       0.0011345384625932814; add:  0.0008455575109275976; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000128175; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.0004085092553125001; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.00019645035749999996; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0003811410199921876; add:  0.00007717500000000002; add:  0.0012464304642064259; add: 
       0.00020837250000000002; add:  0.0007725661289884177; add:  0.0005878685803759568; add: 
       0.00015865403437500001; add:  0.00011402250000000001; add:  0.00017303268749999998; add: 
       0.0001828725; add:  0.00009885000000000002; add:  0.00012052500000000002; add: 
       0.00010267500000000002; add:  0.0002678228812500001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00023969287500000004; add:  0.0008607545337244333; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00009885000000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00026326623179687505; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0007833576583764941; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.0005110525401105469; add: 
       0.00019871976984375; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0003483386168671875; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00042438202159986323; add: 
       0.00019396244308710937; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00019698280387499996; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.00014299384687500004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00047081371679001953; add: 
       0.000030000000000000004; add:  0.0009089884096401953; add:  0.000030000000000000004; add: 
       0.000224376386859375; add:  0.00011402250000000001; add:  0.00005550000000000001; add: 
       0.0009667603782073827; add:  0.000030000000000000004; add:  0.00011125893750000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.001241030682654688; add: 
       0.000030000000000000004; add:  0.0004823641678046876; add:  0.00009559875000000003; add: 
       0.00021509509687500002; add:  0.00034904545746992195; add:  0.0008725268335975779.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:37'!
privExRank18: exp
	exp add:
             0.000030000000000000004; add:  0.0004102900824509962; add:  0.0002089040241216797; add: 
       0.000030000000000000004; add:  0.0003740019745820314; add:  0.0012997076386943947; add: 
       0.00005550000000000001; add:  0.00018994875000000002; add:  0.0006884943091336133; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00012435000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0011576753198899027; add: 
       0.000030000000000000004; add:  0.000649733662027949; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000492883398434199; add:  0.0007212162063932225; add: 
       0.000128175; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00036528014310937504; add:  0.0009957260430857229; add:  0.000030000000000000004; add: 
       0.001329195085524004; add:  0.0006927975738414062; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.001100317863523008; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000146025; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.0006030464054154491; add: 
       0.00007717500000000002; add:  0.00014277375000000002; add:  0.00011727375000000003; add: 
       0.000030000000000000004; add:  0.0008181163542995311; add:  0.000030000000000000004; add: 
       0.00014277375000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0014100132311993948; add:  0.00010267500000000002; add:  0.00009559875000000003; add: 
       0.0006551366364775194; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.00008100000000000002; add:  0.00010267500000000002; add:  0.00005550000000000001; add: 
       0.0007480018415248242; add:  0.0005355432135937501; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00027271537500000005; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00046871357410369156; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000128175; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00012968268750000001; add: 
       0.00013675893750000003; add:  0.0006451623310560742; add:  0.000030000000000000004; add: 
       0.00020930038125; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00025474342050342774; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0006523670799398827; add:  0.000030000000000000004; add:  0.00014919574171875; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00009559875000000003; add: 
       0.000204767596875; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.00009885000000000002; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.0004906871529635349; add: 
       0.000030000000000000004; add:  0.00014277375000000002; add:  0.000030000000000000004; add: 
       0.00030763942364748055; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0001677; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00008100000000000002; add:  0.00014810643750000002; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.0004704629513872852; add: 
       0.000205829034375; add:  0.00009885000000000002; add:  0.00019853268750000003; add: 
       0.00012968268750000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00013588458234375003; add:  0.000030000000000000004; add:  0.0007767642435570312; add: 
       0.000030000000000000004; add:  0.0006168027873406638; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.00023329875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.0005132322826406251; add:  0.0005522987924888865; add:  0.00009885000000000002; add: 
       0.00009885000000000002; add:  0.0002266018125; add:  0.0015834187107174223; add: 
       0.00033898444402460947; add:  0.0008324012817950196; add:  0.000030000000000000004; add: 
       0.000180329034375; add:  0.0008978864941825587; add:  0.00005550000000000001; add: 
       0.00014299384687500004; add:  0.0001801089375; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0005692153571463084; add:  0.0005368253732539843; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0006468409577320312; add:  0.000030000000000000004; add:  0.0003211811310781251; add: 
       0.000030000000000000004; add:  0.0006582275594396678; add:  0.0007064921391304686; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010650000000000001; add: 
       0.001123349208692012; add:  0.00013894875000000002; add:  0.00017174509687500002; add: 
       0.000768254994630996; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.001407102531453867; add:  0.000030000000000000004; add: 
       0.0009750694000490038; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00024731642339179686; add:  0.00015154476984375004; add:  0.000030000000000000004; add: 
       0.00035790707901488293; add:  0.00022954649988632815; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00027896925010724617; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00037515426984375004; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.0009195100252817186; add:  0.00010267500000000002; add:  0.0011678366105352931; add: 
       0.0004596703178347658; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0004559106924523828; add:  0.000030000000000000004; add:  0.0001823163804609375; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.001121086223968906; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00004275000000000001; add: 
       0.00007717500000000002; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00014534287500000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:37'!
privExRank19: exp
	exp add:
             0.0017556940398932423; add:  0.00015007009687500002; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000168493846875; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000636605465767539; add:  0.0006222834791867967; add:  0.000030000000000000004; add: 
       0.00016225893750000003; add:  0.00020495467921875; add:  0.00019540503998437497; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00009559875000000003; add: 
       0.00008100000000000002; add:  0.000163766625; add:  0.0004007745255792969; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00023427967921875; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00040445321618064444; add: 
       0.0002949046588605469; add:  0.00052845754028125; add:  0.000030000000000000004; add: 
       0.0004532404760482618; add:  0.00028161129623437505; add:  0.00023373894375000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0012445951359051468; add: 
       0.000030000000000000004; add:  0.000168493846875; add:  0.00057090017059375; add: 
       0.000030000000000000004; add:  0.0005946650262327733; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.0001973778608917969; add:  0.0004028958548752149; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0004724473638984376; add:  0.00007717500000000002; add:  0.00025497271615529296; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0003436367774209181; add:  0.0006515472775947068; add:  0.000030000000000000004; add: 
       0.0007925057115101563; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00025811662500000007; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0003093791396384961; add: 
       0.000030000000000000004; add:  0.0004234529288368946; add:  0.0005038604170281249; add: 
       0.00012109875000000002; add:  0.00010267500000000002; add:  0.0001784913804609375; add: 
       0.0006189891913622458; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00010650000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0010110427299035549; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.0012826222617107622; add: 
       0.0009419121841931444; add:  0.00010650000000000001; add:  0.00005550000000000001; add: 
       0.00013293393750000003; add:  0.0005449953795705271; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00009885000000000002; add:  0.0012708980017750392; add: 
       0.00005550000000000001; add:  0.00020881269375; add:  0.00035306008668643563; add: 
       0.000030000000000000004; add:  0.0005407651450046874; add:  0.0001356975; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00022511452490337893; add: 
       0.000030000000000000004; add:  0.00021046892339179687; add:  0.00043287358488302733; add: 
       0.000030000000000000004; add:  0.00019783027718359373; add:  0.000030000000000000004; add: 
       0.00031611087217968756; add:  0.00005550000000000001; add:  0.0003520324635937501; add: 
       0.00011125893750000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0004311003574218751; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003175084025693555; add:  0.0011122127446247852; add: 
       0.00012435000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0009516672924987891; add:  0.0007951100024036912; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0004405921018214845; add: 
       0.0006293696852640232; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00014277375000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00011125893750000003; add: 
       0.000221269125; add:  0.0002654379938572461; add:  0.00005550000000000001; add: 
       0.00010650000000000001; add:  0.000030000000000000004; add:  0.00015154476984375004; add: 
       0.00010267500000000002; add:  0.0003013663127651465; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00005550000000000001; add:  0.00009885000000000002; add: 
       0.000030000000000000004; add:  0.00028801585312500014; add:  0.00051896699171875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00007717500000000002; add: 
       0.00027813865045312505; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002295803240625; add:  0.000591690735639453; add: 
       0.00040644273358593757; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0004418071279275587; add:  0.00005550000000000001; add:  0.0004835387763812108; add: 
       0.001059485402328809; add:  0.000998035093129512; add:  0.00012052500000000002; add: 
       0.00010267500000000002; add:  0.0007237203894777343; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.00021695643750000001; add:  0.00007717500000000002; add: 
       0.00012968268750000001; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0022970199819720605; add:  0.00005550000000000001; add: 
       0.00047827658859375005; add:  0.00005550000000000001; add:  0.00006825; add: 
       0.0005510104412360739; add:  0.0010187398686616798; add:  0.00011125893750000003.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:37'!
privExRank20: exp
	exp add:
             0.00005550000000000001; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.00012968268750000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00013293393750000003; add:  0.00017321976984375; add: 
       0.00016225893750000003; add:  0.00019145643750000002; add:  0.00007717500000000002; add: 
       0.00031651012275117196; add:  0.00014659875000000002; add:  0.00039644005107951184; add: 
       0.0002660132896875; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000152419125; add:  0.0016891438247597268; add:  0.00012457009687500003; add: 
       0.00016119750000000003; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.0009333136792756053; add:  0.000030000000000000004; add: 
       0.00019528143750000004; add:  0.00009885000000000002; add:  0.00005550000000000001; add: 
       0.0003461528180871094; add:  0.000030000000000000004; add:  0.00039424190589769544; add: 
       0.00033577812302578136; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.0002877893380792969; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002240326875; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.0005661583932553319; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0010921575166542774; add:  0.0014712415030663285; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00023186717123437498; add:  0.0011153109768279107; add: 
       0.00037087997203593765; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0005527790258255663; add: 
       0.00008100000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000128175; add: 
       0.00005550000000000001; add:  0.00009559875000000003; add:  0.0003304736308681447; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.0018335073900723833; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.0001513576875; add:  0.00032466091776349617; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00022403268749999998; add:  0.0006712320912878907; add: 
       0.00024917903437500006; add:  0.000030000000000000004; add:  0.00025780846905468756; add: 
       0.000030000000000000004; add:  0.0008138743728837696; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005576755996828122; add:  0.0006588155804808006; add: 
       0.00007717500000000002; add:  0.0005866007615345508; add:  0.00005550000000000001; add: 
       0.00029290500012617196; add:  0.0007023281972303903; add:  0.000030000000000000004; add: 
       0.0015980741978907033; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00005550000000000001; add:  0.00016827375000000002; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0009396548305022267; add: 
       0.00012109875000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.001814793173989063; add:  0.000030000000000000004; add: 
       0.00009559875000000003; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.002758934391622441; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0003839836037885841; add:  0.000030000000000000004; add: 
       0.0008133136555286917; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.00038753493443324235; add: 
       0.000030000000000000004; add:  0.00019609428398671875; add:  0.001827727958064629; add: 
       0.0007766777549231443; add:  0.002192770461927129; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0006964689514820309; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.0001356975; add: 
       0.000789890542953125; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00021036787500000001; add:  0.000030000000000000004; add:  0.00016444875000000002; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.00017146863914296875; add: 
       0.00009559875000000003; add:  0.00043470567059683606; add:  0.000209654034375; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0002679983892566408; add:  0.000030000000000000004; add: 
       0.000172545; add:  0.00012109875000000002; add:  0.00024865833234375003; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00015007009687500002; add:  0.00005550000000000001; add: 
       0.0005220628684277733; add:  0.0007966927069299414; add:  0.00018691759687499999; add: 
       0.0008192211742001366; add:  0.000030000000000000004; add:  0.0011461627584204495; add: 
       0.00012435000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00017303268749999998; add:  0.0005645964837343749; add:  0.0003273738042187501; add: 
       0.00013293393750000003; add:  0.000030000000000000004; add:  0.0011958616266875002; add: 
       0.00011727375000000003; add:  0.0006563778608917968; add:  0.00038789903437500006; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00014277375000000002; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.0003000125285068556; add:  0.00009885000000000002; add:  0.0001945941646875; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0001611975; add:  0.000030000000000000004; add:  0.00019422; add:  0.00005550000000000001.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:38'!
privExRank21: exp
	exp add:
             0.000030000000000000004; add:  0.000030000000000000004; add:  0.0003215274082826368; add: 
       0.00009559875000000003; add:  0.00008100000000000002; add:  0.0005578175908965235; add: 
       0.00007717500000000002; add:  0.00010650000000000001; add:  0.00007717500000000002; add: 
       0.0002650830054609375; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.0003952191616555079; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.00010650000000000001; add:  0.00030403575000000006; add:  0.00020817806250000002; add: 
       0.00011125893750000003; add:  0.0007993951805930079; add:  0.0009647913134157617; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000140230284375; add:  0.00015843393750000003; add: 
       0.0001573725; add:  0.00022963383274453124; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00005550000000000001; add:  0.00012109875000000002; add:  0.0005682106623220115; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00048354904214099615; add: 
       0.000171525; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0011088292365873393; add: 
       0.00005550000000000001; add:  0.00019399384687500003; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000478586195888496; add:  0.000030000000000000004; add:  0.00027036634687500004; add: 
       0.00013293393750000003; add:  0.000985747305325; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000187405284375; add:  0.00009559875000000003; add:  0.00037878833859375006; add: 
       0.000030000000000000004; add:  0.00025573217936718755; add:  0.000030000000000000004; add: 
       0.0004222222140789454; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.00009885000000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0008828566442090232; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000126919125; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0008706173606547655; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.001097905835669102; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0012911886927233205; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0014471317232076367; add:  0.00007717500000000002; add:  0.00032919842311718756; add: 
       0.0006736807611742187; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0005083643571588086; add:  0.00020877967921875; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000243164221875; add:  0.000267602784375; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00047918890923906253; add: 
       0.000030000000000000004; add:  0.0004517866978943946; add:  0.0015112480414817186; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00021090620749218748; add:  0.0005817136509740037; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0006902496070234765; add:  0.00005550000000000001; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00023004750000000002; add:  0.000030000000000000004; add: 
       0.00014810643750000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0006170154330910936; add:  0.000165730284375; add:  0.00020927027636835936; add: 
       0.00042570324346896486; add:  0.0005386219151164453; add:  0.00007717500000000002; add: 
       0.0013819125144174025; add:  0.000030000000000000004; add:  0.0010520113375331644; add: 
       0.00009559875000000003; add:  0.0005797662982845508; add:  0.00015007009687500002; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00026522826441796886; add: 
       0.0001356975; add:  0.00009885000000000002; add:  0.0012027005989846294; add: 
       0.00005550000000000001; add:  0.00017685768750000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00016119750000000003; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.00015755958234375; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0010036956054313869; add: 
       0.0001866975; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0004468573575634569; add:  0.0007568851183328513; add: 
       0.00042272894573730486; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0003257730341351564; add:  0.0005692704913527343; add:  0.00019777118175802734; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0002544981050078126; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.0009998740718150587; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00028684279252365243; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00022644259687500004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.00029678490030468754; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00090625993512791; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00021531519375.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:38'!
privExRank22: exp
	exp add:
             0.0004893327048039453; add:  0.00008100000000000002; add:  0.00028449085474904296; add: 
       0.00011402250000000001; add:  0.000030000000000000004; add:  0.0004426083352713087; add: 
       0.000030000000000000004; add:  0.000168493846875; add:  0.000225540375; add: 
       0.00024176661889201175; add:  0.00010650000000000001; add:  0.00038301536889201183; add: 
       0.0003264409174306447; add:  0.000030000000000000004; add:  0.0007187389780083399; add: 
       0.0006355851256347263; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000944545681565801; add:  0.00044651261407503907; add:  0.00011727375000000003; add: 
       0.00008100000000000002; add:  0.0005137126510087695; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.0001392675; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00014277375000000002; add:  0.00020859259687499998; add: 
       0.0001806826875; add:  0.00009559875000000003; add:  0.00012109875000000002; add: 
       0.000030000000000000004; add:  0.00006825; add:  0.00011125893750000003; add: 
       0.00011402250000000001; add:  0.00021219750000000002; add:  0.00005550000000000001; add: 
       0.0005975487986158006; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.0006294703299214255; add:  0.00010267500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00024502556250000004; add:  0.000030000000000000004; add: 
       0.00021736361671874997; add:  0.000030000000000000004; add:  0.000478267900330039; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005636191341747459; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0001610030625; add: 
       0.0003478678864488282; add:  0.000030000000000000004; add:  0.0008226126343261718; add: 
       0.00044772343749357427; add:  0.0010844268495790823; add:  0.00027328157421093753; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.0004377152188212696; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.0003810599095457033; add: 
       0.000030000000000000004; add:  0.0007371604510937501; add:  0.0010461534389142773; add: 
       0.00024611486671875007; add:  0.0008474182917279883; add:  0.00038903171229613285; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.0007478200571626757; add:  0.000030000000000000004; add:  0.00013293393750000003; add: 
       0.0003705205402589845; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.00015007009687500002; add:  0.00026467378371210936; add:  0.00013330032249609376; add: 
       0.00047262199203455097; add:  0.000168493846875; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00019035592921875; add: 
       0.00005550000000000001; add:  0.0004023563457193946; add:  0.000792128087533164; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0004883296733917968; add: 
       0.00005550000000000001; add:  0.0004559052944855469; add:  0.000497331361295918; add: 
       0.00005550000000000001; add:  0.000400097954671875; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.00007717500000000002; add: 
       0.0001513576875; add:  0.000030000000000000004; add:  0.00011402250000000001; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.0005869410960447462; add: 
       0.00005550000000000001; add:  0.00011727375000000003; add:  0.00014985; add: 
       0.00046397884894101556; add:  0.00013293393750000003; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0004985444978679295; add: 
       0.00009559875000000003; add:  0.00029019763671093757; add:  0.00024614788125; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00004275000000000001; add:  0.00005550000000000001; add:  0.0002890909435165822; add: 
       0.000030000000000000004; add:  0.0009232520230490042; add:  0.00007717500000000002; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.0002699518125; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.0004820662698785156; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0001584339375; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00011727375000000003; add:  0.0004580961996339844; add:  0.00005550000000000001; add: 
       0.0034371926123786135; add:  0.0007171233431230858; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0004940070602542576; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0020155847563304292; add:  0.00026344306061718757; add: 
       0.00025103522718749997; add:  0.00047311409130681634; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.0005740974556124608; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0003655747524131056; add:  0.0003691095187500001; add: 
       0.000128175; add:  0.0002538039375000001; add:  0.000185441625; add:  0.0004509094982433595; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.00012968268750000001; add:  0.00005550000000000001; add:  0.0013434953179166213; add: 
       0.00019364019375; add:  0.00008100000000000002; add:  0.000152419125; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00036471634687500003; add:  0.000030000000000000004; add:  0.00017557009687500002; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00009559875000000003; add: 
       0.00016225893750000003; add:  0.000140230284375; add:  0.0010686188597835936; add: 
       0.0005728017023122849; add:  0.000030000000000000004; add:  0.00028052439499218754; add: 
       0.0003154114045719142; add:  0.0003410776875000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.00007717500000000002; add:  0.000585874040565371; add: 
       0.0008652517462578516; add:  0.0006117752634263864; add:  0.0006415838093136718; add: 
       0.00005550000000000001; add:  0.000987794997580957; add:  0.00007717500000000002; add: 
       0.0003370707326484376; add:  0.00007717500000000002; add:  0.00007717500000000002; add: 
       0.0011201223724816604; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00041637400038828116.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:38'!
privExRank23: exp
	exp add:
             0.00030097138671093755; add:  0.0005901878716493943; add:  0.0009725682330679693; add: 
       0.0005054563544739061; add:  0.00008100000000000002; add:  0.0001744732418671875; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0011511606224924999; add: 
       0.00005550000000000001; add:  0.00017469574171875; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00008100000000000002; add: 
       0.00007717500000000002; add:  0.00017174509687500002; add:  0.0005585642724306445; add: 
       0.0004331871046195312; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.0006780410787286913; add:  0.00019145643750000002; add:  0.0006755674858472655; add: 
       0.0004212642249228322; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00020085; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00011727375000000003; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00017730948300342773; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00012109875000000002; add:  0.000030000000000000004; add: 
       0.0009763612694554688; add:  0.00024463889484375003; add:  0.00021184384687500003; add: 
       0.000030000000000000004; add:  0.00013675893750000003; add:  0.00007717500000000002; add: 
       0.000030000000000000004; add:  0.00036788708859375; add:  0.0012347261757795606; add: 
       0.000030000000000000004; add:  0.00023885153437500003; add:  0.0002056089375; add: 
       0.00020476759687499999; add:  0.000030000000000000004; add:  0.0009038579981077738; add: 
       0.00020235768750000003; add:  0.000030000000000000004; add:  0.00020859259687500004; add: 
       0.00005550000000000001; add:  0.00036820402530468756; add:  0.00005550000000000001; add: 
       0.00066419930620291; add:  0.00010267500000000002; add:  0.0003713825254372853; add: 
       0.00005550000000000001; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.0004132730735068556; add:  0.00011402250000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000610537969854375; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00005550000000000001; add:  0.00005550000000000001; add:  0.00231323575590872; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0007635236087597262; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00012109875000000002; add: 
       0.00008100000000000002; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0006835465769273435; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.0002674281846796875; add: 
       0.00089338520952625; add:  0.00007717500000000002; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00010267500000000002; add:  0.001359938627840293; add:  0.00008100000000000002; add: 
       0.00014534287500000002; add:  0.00005550000000000001; add:  0.00010267500000000002; add: 
       0.000030000000000000004; add:  0.00008100000000000002; add:  0.00030859339458498054; add: 
       0.00025607358488302737; add:  0.000030000000000000004; add:  0.00040411500814865246; add: 
       0.000030000000000000004; add:  0.00012968268750000001; add:  0.000030000000000000004; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.00034456086915078134; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00010267500000000002; add: 
       0.00011727375000000003; add:  0.00009559875000000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.00012109875000000002; add:  0.00013293393750000003; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00011125893750000003; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0007807470633976951; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.00005550000000000001; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0003173643442031251; add:  0.00009559875000000003; add: 
       0.000030000000000000004; add:  0.00010650000000000001; add:  0.000030000000000000004; add: 
       0.0006441722416652344; add:  0.00024267653437499997; add:  0.000030000000000000004; add: 
       0.00008100000000000002; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00011125893750000003; add:  0.000030000000000000004; add: 
       0.00016827375000000002; add:  0.0002450773350314649; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005768196484567382; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0014809880060556444; add: 
       0.0003982600444914454; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.00007717500000000002; add:  0.0006242420395359766; add:  0.000030000000000000004; add: 
       0.00016444875; add:  0.000030000000000000004; add:  0.0003311918156250001; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0008059152801275586; add:  0.0012004930497145903; add: 
       0.000030000000000000004; add:  0.0004951419985664452; add:  0.0004380695932937892; add: 
       0.000030000000000000004; add:  0.00010267500000000002; add:  0.00011727375000000003; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.0005296988262009961; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00010650000000000001; add:  0.000030000000000000004; add:  0.0005620807610697853; add: 
       0.0005286613671070311; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004.
! !

!SomPageRank class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 21:39'!
privExRank24: exp
	exp add:
       0.00011125893750000003; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000760817167551992; add:  0.000030000000000000004; add:  0.00010650000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0003138477416337697; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.0008371103585068553; add:  0.00007717500000000002; add: 
       0.0004518288427982617; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0002608801875; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000164668846875; add:  0.00011727375000000003; add:  0.0003858458493481542; add: 
       0.00007717500000000002; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.001094358015352618; add:  0.00035263408031250003; add: 
       0.000370735670154336; add:  0.00008100000000000002; add:  0.0007597855260648826; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00009885000000000002; add: 
       0.00005550000000000001; add:  0.00014277375000000002; add:  0.00012457009687500003; add: 
       0.00005550000000000001; add:  0.00016444875000000002; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.00007717500000000002; add:  0.00009559875000000003; add: 
       0.0004551034628453516; add:  0.00017174509687500002; add:  0.00016119750000000003; add: 
       0.00005550000000000001; add:  0.000030000000000000004; add:  0.0003727185005080275; add: 
       0.00012435000000000001; add:  0.00048311825031271483; add:  0.000030000000000000004; add: 
       0.0012598353609723147; add:  0.000030000000000000004; add:  0.000030000000000000004; add: 
       0.00038769586070216803; add:  0.00005550000000000001; add:  0.00010267500000000002; add: 
       0.00013588458234375003; add:  0.000568914242375215; add:  0.0006243928897521678; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00007717500000000002; add: 
       0.0010377028924124902; add:  0.00008100000000000002; add:  0.00005550000000000001; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.0005503465372175779; add: 
       0.000030000000000000004; add:  0.001355548606467051; add:  0.0003556893808617188; add: 
       0.000030000000000000004; add:  0.00016827375; add:  0.000030000000000000004; add: 
       0.0008246139931943555; add:  0.000030000000000000004; add:  0.0010154740938932223; add: 
       0.00040518822771779304; add:  0.00045871095761835937; add:  0.00005550000000000001; add: 
       0.00016701787500000002; add:  0.002071953365629297; add:  0.000030000000000000004; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.000030000000000000004; add:  0.0005470457547568554; add: 
       0.0005157095790148827; add:  0.000030000000000000004; add:  0.000688324568213281; add: 
       0.0006465114741332422; add:  0.0010292594947507033; add:  0.00005550000000000001; add: 
       0.000030000000000000004; add:  0.00005550000000000001; add:  0.0001649910962121094; add: 
       0.000030000000000000004; add:  0.00009559875000000003; add:  0.00014277375000000002; add: 
       0.00010267500000000002; add:  0.000030000000000000004; add:  0.00031039666483593755; add: 
       0.0005295671026200583; add:  0.000030000000000000004; add:  0.00008100000000000002; add: 
       0.0004951876449921876; add:  0.000146025; add:  0.0005186043726941406; add: 
       0.00005550000000000001; add:  0.00007717500000000002; add:  0.00011727375000000003; add: 
       0.0004914393382277344; add:  0.00030981865964960947; add:  0.0005172251503359373; add: 
       0.00007717500000000002; add:  0.00014624509687500003; add:  0.0009398635399813478; add: 
       0.0014674101670753515; add:  0.00005550000000000001; add:  0.000030000000000000004; add: 
       0.000030000000000000004.
! !

!SomPageRank methodsFor: 'pageranking' stamp: 'StefanMarr 4/5/2015 21:13'!
checkBasedOnFirstResult: pageRanks
    pageRanks size = innerIterations ifFalse: [ ^ false ].

    firstResult == nil ifTrue: [
      firstResult := pageRanks.
      ^ true
    ] ifFalse: [
      1 to: innerIterations do: [:i |
        (firstResult at: i) = (pageRanks at: i) ifFalse: [
          ^ false
        ]
      ]
    ].
    
    ^ true! !

!SomPageRank methodsFor: 'pageranking' stamp: 'StefanMarr 4/5/2015 21:43'!
generateRandomPagesN: n outLinks: outLinks divisor: divisor
    " generates an array of random pages and their links "
    | pages |
    " matrix cell i,j means link from j->i "
    pages := Array new: n * n withAll: 0.
    
    0 to: n - 1 do: [:i |
      outLinks at: i + 1 put: 0.

      0 to: n - 1 do: [:j |
        (i ~= j and: [(SomJenkinsRandom random abs rem: divisor) = 0]) ifTrue: [
          pages at: (i * n + j) + 1 put: 1.
          outLinks at: i + 1 put: (outLinks at: i + 1) + 1
        ]
      ].

    " the case with no outlinks is afunctioned "
    (outLinks at: i + 1) = 0 ifTrue: [
        | k |
        k := SomJenkinsRandom random abs rem: n.
        [i = k] whileTrue: [
          k := SomJenkinsRandom random abs rem: n.
        ].

        pages at: (i * n + k) + 1 put: 1.
        outLinks at: i + 1 put: 1
      ]
    ].
    ^ pages! !

!SomPageRank methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 21:11'!
innerBenchmarkLoop
    SomJenkinsRandom seed: 49734321.
    "standard for verification"
    "n := 5000
     iter    := 10.
     thresh  := 0.00000001.
     divisor := 100000."
    ^ self verifyResult: (self runPageRankN: innerIterations iter: 10 thresh: 0.00000001 divisor: 100000).
! !

!SomPageRank methodsFor: 'pageranking' stamp: 'StefanMarr 4/5/2015 21:10'!
mapPageRanks: pages pageRanks: pageRanks maps: maps outLinks: outLinks n: n
    0 to: n - 1 do: [:i |
      | outboundRank |
      outboundRank := (pageRanks at: i + 1) / (outLinks at: i + 1).
      
      0 to: n - 1 do: [:j |
        maps at: (i * n + j) + 1
            put: ((pages at: (i * n + j) + 1) = 0
                   ifTrue:  [0.0]
                   ifFalse: [(pages at: (i * n + j) + 1) * outboundRank])
      ]
    ]! !

!SomPageRank methodsFor: 'pageranking' stamp: 'StefanMarr 4/5/2015 21:11'!
reducePageRanks: pageRanks maps: maps n: n
    | dif |
    dif := 0.0.

    0 to: n - 1 do: [:j |
      | oldRank newRank |
      oldRank := pageRanks at: j + 1.
      newRank := 0.0.
      
      0 to: n - 1 do: [:i |
        newRank := newRank + (maps at: (i * n + j) + 1)
      ].

      newRank := ((1 - SomPageRank DFactor) / n) + (SomPageRank DFactor * newRank).
      dif := (newRank - oldRank) abs > dif
               ifTrue: [(newRank - oldRank) abs] ifFalse: [ dif ].
      pageRanks at: j + 1 put: newRank
    ].
    ^ dif! !

!SomPageRank methodsFor: 'pageranking' stamp: 'StefanMarr 4/5/2015 21:43'!
runPageRankN: n iter: iter thresh: thresh divisor: divisor
    | nbLinks maxDiff pageRanks maps outLinks t pages |
    maxDiff   := Float infinity.
    pageRanks := Array new: n     withAll: 1.0 / n.
    maps      := Array new: n * n withAll: 0.0.
    outLinks  := Array new: n     withAll: 0.

    pages := self generateRandomPagesN: n outLinks: outLinks divisor: divisor.

    nbLinks := 0.
    0 to: n - 1 do: [:i |
      1 to: n do: [:j |
        nbLinks := nbLinks + (pages at: i * n + j)
      ]
    ].

    t := 1.
    [t <= iter and: [maxDiff >= thresh]] whileTrue: [
      self mapPageRanks: pages pageRanks: pageRanks maps: maps outLinks: outLinks n: n.
      maxDiff := self reducePageRanks: pageRanks maps: maps n: n.
      t := t + 1
    ].
    
    ^ pageRanks! !

!SomPageRank methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 21:57'!
verifyResult: pageRanks
    innerIterations = 5000 " and: [iter = 10 and: [thresh = 0.00000001 and: [divisor = 100000]]]"
      ifTrue: [
        | exp |
        exp := SomPageRank ExpectedPageRanks.
        pageRanks size = exp size ifFalse: [
          self error: 'Invalid length of page_ranks array'
        ].
      
        1 to: pageRanks size do: [:i | 
          (pageRanks at: i) = (exp at: i) ifFalse: [
            self error: 'ERROR: page_ranks[' + i asString + ']=' +
                        (pageRanks at: i) + ' differs from the expected value: ' +
                        (exp at: i)
          ]
        ] ]
      ifFalse: [
        ^ self checkBasedOnFirstResult: pageRanks
      ].
    ^ true! !

SomBenchmark subclass: #SomPermute
	instanceVariableNames: 'count v'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomPermute methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:25'!
benchmark
        count := 0.
        v     := Array new: 7.
        self permute: 6.
        ^ count! !

!SomPermute methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:26'!
permute: n
        count := count + 1.
        (n ~= 0)
            ifTrue: [
                self permute: n - 1.
                n to: 1 by: -1 do: [ :i |
                    self swap: n with: i.
                    self permute: n - 1.
                    self swap: n with: i ] ]! !

!SomPermute methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 14:26'!
swap: i with: j
        | tmp |
        tmp := v at: i.
        v at: i put: (v at: j).
        v at: j put: tmp! !

!SomPermute methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:26'!
verifyResult: result
      ^ self assert: 8660 equals: result! !

SomBenchmark subclass: #SomQueens
	instanceVariableNames: 'freeMaxs freeRows freeMins queenRows'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomQueens methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:28'!
benchmark
        | result |
        result := true.
        1 to: 10 do: [ :i | result := result and: self queens ].
        ^ result! !

!SomQueens methodsFor: 'queens' stamp: 'StefanMarr 4/5/2015 14:28'!
placeQueen: c
        1 to: 8 do: [ :r | 
            (self row: r column: c)
                ifTrue: [
                    queenRows at: r put: c.
                    self row: r column: c put: false.
                    (c = 8) ifTrue: [ ^true ].
                    (self placeQueen: c + 1) ifTrue: [ ^true ].
                    self row: r column: c put: true ] ].
        ^false! !

!SomQueens methodsFor: 'queens' stamp: 'StefanMarr 4/5/2015 14:28'!
queens
        freeRows  := Array new:  8 withAll: true.
        freeMaxs  := Array new: 16 withAll: true.
        freeMins  := Array new: 16 withAll: true.
        queenRows := Array new:  8 withAll: -1.
        ^ self placeQueen: 1! !

!SomQueens methodsFor: 'queens' stamp: 'StefanMarr 4/5/2015 14:28'!
row: r column: c
        ^ (freeRows at: r) & (freeMaxs at: c + r) & (freeMins at: c - r + 8)
! !

!SomQueens methodsFor: 'queens' stamp: 'StefanMarr 4/5/2015 14:29'!
row: r column: c put: v
        freeRows at: r         put: v.
        freeMaxs at: c + r     put: v.
        freeMins at: c - r + 8 put: v! !

!SomQueens methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:28'!
verifyResult: result
      ^ result! !

SomBenchmark subclass: #SomRichards
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-Richards'!

!SomRichards commentStamp: '<historical>' prior: 0!
Richards is an OS kernel simulation benchmark, originally written in BCPL by Martin Richards. The main focus in Richards is on property access and calling functions and methods.!

!SomRichards methodsFor: 'benchs' stamp: 'StefanMarr 4/3/2015 18:16'!
benchmark

	RichObject initialize.
	^ RichRunner start! !

!SomRichards methodsFor: 'benchs' stamp: 'StefanMarr 4/5/2015 13:31'!
verifyResult: result
	^ result! !

SomBenchmark subclass: #SomSieve
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomSieve methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:30'!
benchmark
        | flags |
        flags  := Array new: 5000.
        ^ self sieve: flags size: 5000.! !

!SomSieve methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:33'!
sieve: flags size: size
        | primeCount |
        primeCount := 0.
        1 to: flags size do: [:i | flags at: i put: true ]. "#putAll:"

        2 to: size do: [ :i |
            (flags at: i - 1)
                ifTrue: [
                    | k |
                    primeCount := primeCount + 1.
                    k := i + i.
                    [ k <= size ]
                        whileTrue: [
                            flags at: k - 1 put: false. k := k + i ]. ] ].
        ^primeCount! !

!SomSieve methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:30'!
verifyResult: result
      ^ self assert: 669 equals: result! !

SomBenchmark subclass: #SomSort
	instanceVariableNames: 'smallest largest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

SomSort subclass: #SomBubbleSort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomBubbleSort methodsFor: 'initializing' stamp: 'StefanMarr 4/5/2015 14:24'!
dataSize
	^ 130! !

!SomBubbleSort methodsFor: 'sorting' stamp: 'StefanMarr 4/5/2015 14:20'!
sort: array
        array size to: 1 by: -1 do: [ :i |
            1 to: i - 1 do: [ :j |
                | current next |
                current := array at: j.
                next    := array at: j + 1.
                (current > next)
                    ifTrue: [
                        array at: j put: next.
                        array at: j + 1 put: current ] ] ].
        ^ array! !

SomSort subclass: #SomQuickSort
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomQuickSort methodsFor: 'initializing' stamp: 'StefanMarr 4/5/2015 14:24'!
dataSize
	^ 800! !

!SomQuickSort methodsFor: 'sorting' stamp: 'StefanMarr 4/5/2015 14:21'!
sort: array
        self sort: array low: 1 high: self dataSize.
        ^ array! !

!SomQuickSort methodsFor: 'sorting' stamp: 'StefanMarr 4/5/2015 14:23'!
sort: array low: low high: high
        | pivot i j |
        
        pivot := array at: (low + high) // 2.
        i := low.
        j := high.
        [ i <= j ]
            whileTrue: [
                [ (array at: i) < pivot ] whileTrue: [ i := i + 1 ].
                [ pivot < (array at: j) ] whileTrue: [ j := j - 1 ].
                ( i <= j )
                    ifTrue: [
                        | tmp |
                        tmp := array at: i.
                        array at: i put: (array at: j).
                        array at: j put: tmp.
                        i := i + 1.
                        j := j - 1. ] ].
        
        (low < j)  ifTrue: [ self sort: array low: low high: j  ].
        (i < high) ifTrue: [ self sort: array low: i high: high ]
! !

!SomSort methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:14'!
benchmark
        | array |
        array := self randomArray: self dataSize.
        ^ self sort: array! !

!SomSort methodsFor: 'initializing' stamp: 'StefanMarr 4/5/2015 14:25'!
dataSize
	self subclassResponsibility! !

!SomSort methodsFor: 'initializing' stamp: 'StefanMarr 4/5/2015 15:20'!
randomArray: size
        | array |
        SomRandom initialize.
        array := Array new: size.
        1 to: size do: [:i | array at: i put: SomRandom next ].
        smallest := largest := array at: 1.
        array do: [ :elm |
            (elm > largest)  ifTrue: [ largest  := elm ].
            (elm < smallest) ifTrue: [ smallest := elm ]. ].
        ^array! !

!SomSort methodsFor: 'sorting' stamp: 'StefanMarr 4/5/2015 14:14'!
sort: array
	self subclassResponsibility! !

!SomSort methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 15:20'!
verifyResult: array
        ((array at: 1) ~= smallest)
            | ((array at: array size) ~= largest)
                ifTrue: [
						self error: 'Array is not sorted. smallest: ', smallest asString,
									   ' largest: ', largest asString, ' [1]: ', (array at: 1) asString,
									   ' [l]: ', (array at: array length) asString ].
        3 to: (array size) do: [ :i |
            (array at: i - 1) > (array at: i) 
                ifTrue: [
						self error: 'Array is not sorted. [', i asString, ' - 1]: ',
										(array at: i - 1) asString,
										' [', i asString, ']: ', (array at: i) asString]. ].
        ^ true! !

SomBenchmark subclass: #SomStorage
	instanceVariableNames: 'count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomStorage methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:34'!
benchmark
        SomRandom initialize.
        count := 0.
        self buildTreeDepth: 7.
        ^ count! !

!SomStorage methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:36'!
buildTreeDepth: depth
        count := count + 1.
        ^(depth = 1) 
            ifTrue:  [ Array new: (SomRandom next rem: 10 + 1) ]
            ifFalse: [
	             | arr |
	             arr := Array new: 4.
	             1 to: 4 do: [:i | arr at: i put: (self buildTreeDepth: depth - 1).
		          arr ] ]
! !

!SomStorage methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:34'!
verifyResult: result
      ^ self assert: 5461 equals: result! !

SomBenchmark subclass: #SomTowers
	instanceVariableNames: 'piles movesdone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomTowers methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:39'!
benchmark
        piles := Array new: 4.
        self buildTowerAt: 1 disks: 13.
        movesdone := 0.
        self move: 13 disksFrom: 1 to: 2.
        ^ movesdone! !

!SomTowers methodsFor: 'towers' stamp: 'StefanMarr 4/5/2015 14:37'!
buildTowerAt: pile disks: disks
        disks to: 0 by: -1 do: [ :i |
            self pushDisk: (SomTowersDisk new: i) onPile: pile ]! !

!SomTowers methodsFor: 'towers' stamp: 'StefanMarr 4/5/2015 14:38'!
move: disks disksFrom: fromPile to: toPile
        disks = 1
            ifTrue: [ self moveTopDiskFrom: fromPile to: toPile ]
            ifFalse: [ | otherPile |
                otherPile := (6 - fromPile) - toPile.
                self move: disks - 1 disksFrom: fromPile to: otherPile.
                self moveTopDiskFrom: fromPile to: toPile.
                self move: disks - 1 disksFrom: otherPile to: toPile. ]
! !

!SomTowers methodsFor: 'towers' stamp: 'StefanMarr 4/5/2015 14:37'!
moveTopDiskFrom: fromPile to: toPile
        self pushDisk: (self popDiskFrom: fromPile) onPile: toPile.
        movesdone := movesdone + 1.! !

!SomTowers methodsFor: 'towers' stamp: 'StefanMarr 4/5/2015 14:37'!
popDiskFrom: pile
        | top |
        
        top := piles at: pile.
        top isNil
            ifTrue: [
                self error: 'Attempting to remove a disk from an empty pile' ].
        
        piles at: pile put: top next.
        top next: nil.
        ^top! !

!SomTowers methodsFor: 'towers' stamp: 'StefanMarr 4/5/2015 15:23'!
pushDisk: disk onPile: pile
        | top |
        
        top := piles at: pile.
        ((top isNil not) and: [ disk size >= top size ])
            ifTrue: [ self error: 'Cannot put a big disk on a smaller one' ].
        
        disk next: top.
        piles at: pile put: disk.! !

!SomTowers methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 14:39'!
verifyResult: result
      ^ self assert: 8191 equals: result! !

SomBenchmark subclass: #SomWhileLoop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks-LanguageFeatures'!

!SomWhileLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:03'!
benchmark
        | sum |
        sum := 0.
        [sum < 20000]
            whileTrue:
                [sum := sum + self singleRun].
        ^ sum! !

!SomWhileLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:03'!
singleRun
        | sum |
        sum := 0.
        [sum < 1000]
            whileTrue:
                [sum := sum + 1].
        ^ sum! !

!SomWhileLoop methodsFor: 'benchmarking' stamp: 'StefanMarr 4/5/2015 13:03'!
verifyResult: result
      ^ 20000 = result! !

Object subclass: #SomBenchmarkHarness
	instanceVariableNames: 'total runtimes benchmarkClasses numIterations warmUp innerIterations maxRuntime printAll'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomBenchmarkHarness class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:18'!
run: args
	^ self new run: args! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:50'!
benchmarkClasses: classes
	benchmarkClasses := classes! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 11:33'!
collectBenchmarkClasses: args
        | i benchmarks |
        i := 2. "First argument is the BenchmarkHarness"
        
        benchmarks := OrderedCollection new: args size - 1.
        [i <= args size] whileTrue: [
            | sym cls |
            sym := #Som, (args at: i) asSymbol.
            cls := Smalltalk classNamed: sym.
            cls ifNil: [
                "This item does not seem to be a class, so it is most likely
                 one of the other parameters"
                 benchmarkClasses := benchmarks.
                 benchmarks size = 0 ifTrue: [ self error: 'No benchmark found' ].
                 ^ args copyFrom: i to: args size ].
            benchmarks add: cls.
            i := i + 1 ].
        benchmarks size = 0 ifTrue: [ self error: 'No benchmark found' ].

        benchmarkClasses := benchmarks.
        ^ nil! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:58'!
initialize
        total         := 0.
        runtimes      := Dictionary new.
        numIterations := 1.
        warmUp        := 0.
        innerIterations := 1.
        maxRuntime    := nil.
        printAll      := true.! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:51'!
maxRuntime: seconds
	maxRuntime := seconds! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:51'!
numIterations: anInt
	numIterations := anInt! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:50'!
printAll: aBool
	printAll := aBool! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 11:27'!
printTotal
        ScriptConsole << ('Total Runtime: ', total asString, 'us').! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 13:39'!
printUsage
  ScriptConsole << 'TODO [benchmarks] [num-iterations [warm-up [inner-iter]]]'; cr.
  ScriptConsole cr.
  ScriptConsole << '  benchmarks     - list of benchmark class names (e.g.: Queens Fibonacci Dispatch)'; cr.
  ScriptConsole << '  num-iterations - number of times each benchmark is executed, default: 1'; cr.
  ScriptConsole << '  warm-up        - number of times each benchmark is executed before measuring, default: 0'; cr.
  ScriptConsole << '  inner-iter     - number of times each benchmark is executed in an inner loop, which is measured in total, default: 1'; cr.! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 11:33'!
processArguments: args
        | remainingArgs |
        remainingArgs := self collectBenchmarkClasses: args.
        remainingArgs ifNotNil: [
            numIterations := (remainingArgs at: 1) asInteger.
            remainingArgs size > 1 ifTrue: [
                warmUp := (remainingArgs at: 2) asInteger.
                remainingArgs size > 2 ifTrue: [
                    innerIterations := (remainingArgs at: 3) asInteger.
                ].
            ].
        ]! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 13:40'!
reportRun: bench
        ScriptConsole << (bench name, ': iterations=',
						        bench numIterations asString,
						        ' average: ',
						        (bench total / bench numIterations) asString,
						        'us total: ',
						        bench total asString, 'us'); cr! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 11:31'!
run: args
        args size < 2 ifTrue: [ ^ self printUsage ].
        
        self initialize.
        self processArguments: args.

        self runBenchmarks.
        self printTotal.! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 11:25'!
runBenchmarks
        benchmarkClasses do: [ :benchmark |
            | bench |
            bench := benchmark new.
            bench maxRuntime: maxRuntime.
            
            bench run: numIterations with: warmUp and: innerIterations printAll: printAll.
            total := total + bench total.
            runtimes at: benchmark put: bench total.
            
            self reportRun: bench.

            ScriptConsole cr. ]! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:57'!
total
	^ total! !

!SomBenchmarkHarness methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 10:51'!
warmUp: anInt
	warmUp := anInt! !

Object subclass: #SomJenkinsRandom
	instanceVariableNames: ''
	classVariableNames: 'seed'
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomJenkinsRandom class methodsFor: 'converting' stamp: 'StefanMarr 4/5/2015 19:46'!
as32BitSigned: val
	| r |
	r := val bitAnd: 16rffffffff.
	(r bitAnd: 16r80000000) = 0 ifTrue: [ ^ r ].
	^ (16r100000000 - r) * -1
	! !

!SomJenkinsRandom class methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 20:58'!
random
	"Robert Jenkins' 32 bit integer hash function."
	 seed := ((seed       + 16r7ed55d16)       + (seed << 12)).
    seed := ((seed bitXor: 16rc761c23c) bitXor: ((seed bitAnd: 16rffffffff) >> 19)).
    seed := ((seed       + 16r165667B1)       + (seed <<  5)).
    seed := ((seed       + 16rd3a2646c) bitXor: (seed <<  9)).
    seed := ((seed       + 16rfd7046c5)       + (seed <<  3)).
    seed := self as32BitSigned: ((seed bitXor: 16rb55a4f09) bitXor: ((seed bitAnd: 16rffffffff) >> 16)).
    ^ seed! !

!SomJenkinsRandom class methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 20:31'!
randomFirst
	"Robert Jenkins' 32 bit integer hash function."
	 seed := self as32BitSigned: ((seed       + 16r7ed55d16)       + (self as32BitSigned: ((seed bitAnd: 16rffffffff) << 12))).
    seed := self as32BitSigned: ((seed bitXor: 16rc761c23c) bitXor: ((seed bitAnd: 16rffffffff) >> 19)).
    seed := self as32BitSigned: ((seed       + 16r165667B1)       + (self as32BitSigned: ((seed bitAnd: 16rffffffff) <<  5))).
    seed := self as32BitSigned: ((seed       + 16rd3a2646c) bitXor: (self as32BitSigned: ((seed bitAnd: 16rffffffff) <<  9))).
    seed := self as32BitSigned: ((seed       + 16rfd7046c5)       + (self as32BitSigned: ((seed bitAnd: 16rffffffff) <<  3))).
    seed := self as32BitSigned: ((seed bitXor: 16rb55a4f09) bitXor: ((seed bitAnd: 16rffffffff) >> 16)).
    ^ seed! !

!SomJenkinsRandom class methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 20:49'!
randomOpt
	"Robert Jenkins' 32 bit integer hash function."
	 seed := self as32BitSigned: ((seed       + 16r7ed55d16)       + (seed << 12)).
    seed :=                     ((seed bitXor: 16rc761c23c) bitXor: (seed >> 19)).
    seed :=                     ((seed       + 16r165667B1)       + (seed <<  5)).
    seed :=                     ((seed       + 16rd3a2646c) bitXor: (seed <<  9)).
    seed := self as32BitSigned: ((seed       + 16rfd7046c5)       + (seed <<  3)).
    seed := self as32BitSigned: ((seed bitXor: 16rb55a4f09) bitXor: (seed >> 16)).
    ^ seed! !

!SomJenkinsRandom class methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 18:27'!
seed: val
	seed := val! !

Object subclass: #SomRandom
	instanceVariableNames: 'seed'
	classVariableNames: 'random'
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomRandom class methodsFor: 'class initialization' stamp: 'StefanMarr 4/5/2015 13:41'!
initialize
	^ random := SomRandom new! !

!SomRandom class methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 14:00'!
next
	^ self random next! !

!SomRandom class methodsFor: 'instance creation' stamp: 'StefanMarr 4/5/2015 13:41'!
random
	^ random! !

!SomRandom methodsFor: 'initialization' stamp: 'StefanMarr 4/5/2015 13:37'!
initialize
	seed := 74755! !

!SomRandom methodsFor: 'accessing' stamp: 'StefanMarr 4/5/2015 13:38'!
next
	seed := ((seed * 1309) + 13849) & 65535.
	^ seed! !

!SomRandom methodsFor: 'running' stamp: 'StefanMarr 4/5/2015 13:42'!
run
        | fail |
        ScriptConsole << 'Testing random number generator ... '.
        fail := [ ScriptConsole << 'FAILED:'; cr. ^nil ].
        (self next ~= 22896) ifTrue: fail.
        (self next ~= 34761) ifTrue: fail.
        (self next ~= 34014) ifTrue: fail.
        (self next ~= 39231) ifTrue: fail.
        (self next ~= 52540) ifTrue: fail.
        (self next ~= 41445) ifTrue: fail.
        (self next ~=  1546) ifTrue: fail.
        (self next ~=  5947) ifTrue: fail.
        (self next ~= 65224) ifTrue: fail.
        ScriptConsole << 'PASSED'; cr.! !

Object subclass: #SomTowersDisk
	instanceVariableNames: 'size next'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SOM-Benchmarks'!

!SomTowersDisk class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:23'!
new: value
	^ self basicNew size: value! !

!SomTowersDisk methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:22'!
next
	^ next! !

!SomTowersDisk methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:22'!
next: value
	next := value! !

!SomTowersDisk methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:22'!
size
	^ size! !

!SomTowersDisk methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/5/2015 15:22'!
size: value
	size := value! !
SomRandom initialize!
JSJsonLiteral initialize!
NBBody initialize!
DBStrength initialize!
RichObject initialize!
